import * as mobx from 'mobx';
import { isObservable, isComputed, isObservableArray, isObservableSet, isObservableMap } from 'mobx';
import { isUndefined } from '../utils/type-checks.mjs';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const FLOW_GLOBAL = "FDBG";
const CONSOLE_HIGHLIGHT_STYLE = `
  padding: 2px 4px;
  border-radius: 2px;
  background: #389DFF;
  color: white;`;
const CONSOLE_BOLD_STYLE = `font-weight: bold;`;
class Debugger {
  /**
   * Constructs a new Flo.w debugger instance.
   *
   * A new Flo.w debugger will be created automatically if the `enableDebug` option is
   * specified when initializing a Flo.w context.
   *
   * @param context Flo.w context ID.
   */
  constructor(context) {
    this.context = context;
    __publicField(this, "spyDisposer");
    __publicField(this, "watchDisposers", {});
  }
  /**
   * Install a 'watch' for the specified context state property path.
   *
   * @param path Context state property path.
   */
  watch(path) {
    if (!this.watchDisposers[path]) {
      this.watchDisposers[path] = this.context.state.autorun(() => this.logStateValue(path));
    }
  }
  /**
   * Remove a  previously-installed 'watch' for the specified context state property path.
   *
   * @param path Context state property path.
   */
  unwatch(path) {
    if (path) {
      if (this.watchDisposers[path]) {
        this.watchDisposers[path]();
        delete this.watchDisposers[path];
      }
    } else {
      Object.keys(this.watchDisposers).forEach((key) => this.watchDisposers[key]());
      this.watchDisposers = {};
    }
  }
  /**
   * Set a Flo.w context state property.
   *
   * @param path Context state property path
   * @param value New value to set
   */
  set(path, value) {
    this.context.state.set(path, value);
  }
  /**
   * Read the value of a Flo.w context state property.
   *
   * @param keyOrPath Context state property path.
   * @returns The context state property value.
   */
  get(keyOrPath) {
    if (isUndefined(keyOrPath)) {
      keyOrPath = [];
    } else if (typeof keyOrPath === "string") {
      if (keyOrPath.startsWith("$")) {
        keyOrPath = keyOrPath.slice(1);
      }
      return this.get(keyOrPath.split("."));
    }
    let value = this.context.state;
    const path = [...keyOrPath];
    while (path.length > 0) {
      if (value == null) {
        value = void 0;
        break;
      }
      const key = path.shift();
      value = value[key];
    }
    return toJSHelper(value, {});
  }
  /**
   * Enable verbose debugging of all reactive updates.
   */
  spy() {
    this.unspy();
    this.spyDisposer = mobx.spy(this.spyListener.bind(this));
  }
  /**
   * Disable verbose debugging of all reactive updates.
   */
  unspy() {
    if (this.spyDisposer) {
      this.spyDisposer();
      this.spyDisposer = void 0;
    }
  }
  /**
   * Get a registered map instance by ID.
   */
  maps(mapId) {
    if (!mapId) {
      return Array.from(this.context.maps._maps.keys());
    }
    const mapPromise = this.context.maps.getMap(mapId);
    return mapPromise?.value;
  }
  /**
   * Get a registered reactive query by ID.
   */
  queries(queryId) {
    if (!queryId) {
      return Array.from(this.context.queries._queries.keys());
    }
    const queryPromise = this.context.queries.getQuery(queryId);
    return queryPromise?.value;
  }
  spyListener(event) {
    console.log(`%cspy`, CONSOLE_HIGHLIGHT_STYLE, event);
  }
  logStateValue(path) {
    const value = toJSHelper(this.context.state.get(path), { recurseEverything: false });
    console.log(`%cwatch%c %s`, CONSOLE_HIGHLIGHT_STYLE, CONSOLE_BOLD_STYLE, path, value);
  }
}
function installGlobalDebugger(dbg) {
  console.log(
    `%cFlow.w RDF Debbugger`,
    CONSOLE_HIGHLIGHT_STYLE,
    `Enabled for context '${dbg.context.id}' - access using ${FLOW_GLOBAL} in web developer console.`
  );
  const g = typeof global !== "undefined" ? global : window;
  g[FLOW_GLOBAL] = g[FLOW_GLOBAL] || {
    context: dbg.context,
    watch: dbg.watch.bind(dbg),
    unwatch: dbg.unwatch.bind(dbg),
    set: dbg.set.bind(dbg),
    get: dbg.get.bind(dbg),
    spy: dbg.spy.bind(dbg),
    maps: dbg.maps.bind(dbg),
    queries: dbg.queries.bind(dbg),
    unspy: dbg.unspy.bind(dbg)
  };
}
function cache(map, key, value, options) {
  if (options.detectCycles)
    map.set(key, value);
  return value;
}
function getPlainObjectKeys(object) {
  const enumerables = /* @__PURE__ */ new Set();
  for (const key in object)
    enumerables.add(key);
  Object.getOwnPropertySymbols(object).forEach(function(k) {
    if (Object.getOwnPropertyDescriptor(object, k).enumerable)
      enumerables.add(k);
  });
  return Array.from(enumerables);
}
function toJSHelper(source, options, __alreadySeen = null) {
  if (!options.recurseEverything && !isObservable(source))
    return source;
  if (typeof source !== "object")
    return source;
  if (source === null)
    return null;
  if (source instanceof Date)
    return source;
  if (isComputed(source)) {
    try {
      const value = source.get();
      return { computedValue: value };
    } catch (e) {
      return { computedValueException: e.message };
    }
  }
  const detectCycles = options.detectCycles === true;
  if (detectCycles && source !== null && __alreadySeen.has(source)) {
    return __alreadySeen.get(source);
  }
  if (isObservableArray(source) || Array.isArray(source)) {
    const res_1 = cache(__alreadySeen, source, [], options);
    const toAdd = source.map(function(value) {
      return toJSHelper(value, options, __alreadySeen);
    });
    res_1.length = toAdd.length;
    for (let i = 0, l = toAdd.length; i < l; i++)
      res_1[i] = toAdd[i];
    return res_1;
  }
  if (isObservableSet(source) || Object.getPrototypeOf(source) === Set.prototype) {
    if (options.exportMapsAsObjects === false) {
      const res_2 = cache(__alreadySeen, source, /* @__PURE__ */ new Set(), options);
      source.forEach(function(value) {
        res_2.add(toJSHelper(value, options, __alreadySeen));
      });
      return res_2;
    } else {
      const res_3 = cache(__alreadySeen, source, [], options);
      source.forEach(function(value) {
        res_3.push(toJSHelper(value, options, __alreadySeen));
      });
      return res_3;
    }
  }
  if (isObservableMap(source) || Object.getPrototypeOf(source) === Map.prototype) {
    if (options.exportMapsAsObjects === false) {
      const res_4 = cache(__alreadySeen, source, /* @__PURE__ */ new Map(), options);
      source.forEach(function(value, key) {
        res_4.set(key, toJSHelper(value, options, __alreadySeen));
      });
      return res_4;
    } else {
      const res_5 = cache(__alreadySeen, source, {}, options);
      source.forEach(function(value, key) {
        res_5[key] = toJSHelper(value, options, __alreadySeen);
      });
      return res_5;
    }
  }
  const res = cache(__alreadySeen, source, {}, options);
  getPlainObjectKeys(source).forEach(function(key) {
    res[key] = toJSHelper(source[key], options, __alreadySeen);
  });
  return res;
}

export { Debugger, installGlobalDebugger };
