import P from 'parsimmon';
import { PREFIX, BINARY_RIGHT, BINARY_LEFT, TERNARY_RIGHT } from './parser-helpers.mjs';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class ExpressionParser {
  constructor() {
    __publicField(this, "language");
    this.language = this.createLanguage();
  }
  /**
   * Parse an expression
   *
   * @param expr Expression to parse
   */
  parseExpression(expr) {
    return this.language.Expr.parse(expr);
  }
  /**
   * Parse a complete assignment program of the form:
   * a: expr1;
   * b: expr2;
   *
   * @param program Program to parse
   */
  parseProgram(program) {
    return this.language.ProgramOrEmpty.parse(program);
  }
  createLanguage() {
    const operatorTable = [
      { type: PREFIX, ops: this.operators({ "-": "-", "!": "!" }) },
      { type: BINARY_RIGHT, ops: this.operators({ "^": "**" }) },
      { type: BINARY_LEFT, ops: this.operators({ "*": "*", "/": "/", "%": "%" }) },
      { type: BINARY_LEFT, ops: this.operators({ "+": "+", "-": "-" }) },
      { type: BINARY_LEFT, ops: this.operators({ ">=": ">=", "<=": "<=" }) },
      { type: BINARY_LEFT, ops: this.operators({ "<": "<", ">": ">" }) },
      {
        type: BINARY_LEFT,
        ops: this.operators({ regexi: "~*", equali: "=*", "==": "==", "!=": "!=", "~": "~" })
      },
      { type: BINARY_LEFT, ops: this.operators({ all: "&&" }) },
      { type: BINARY_LEFT, ops: this.operators({ any: "||" }) },
      { type: TERNARY_RIGHT, ops: this.operators({ case: "?" }) }
    ];
    return P.createLanguage({
      Whitespace: () => P.regexp(/[ \n]*/).desc("whitespace"),
      NotNewline: () => P.regexp(/[^\n]*/).desc("non-newline"),
      LineComment: (r) => r.NotNewline.wrap(P.string("//"), P.alt(P.string("\n"), P.eof)),
      BlockComment: () => P.regexp(/\/\*[\s\S]*?\*\//),
      Comment: (r) => P.alt(r.LineComment, r.BlockComment).desc("comment"),
      End: () => P.alt(P.string(";"), P.eof),
      _: (r) => r.Comment.sepBy(r.Whitespace).trim(r.Whitespace),
      Integer: () => {
        return P.regexp(/-?[0-9]+/).map(Number);
      },
      Decimal: () => {
        return P.regexp(/-?[0-9]+\.[0-9]+/).map(Number);
      },
      Number: (r) => {
        return P.alt(r.Decimal, r.Integer).desc("number");
      },
      Boolean: () => {
        return P.alt(P.string("true"), P.string("false")).map((val) => val === "true").desc("boolean");
      },
      Null: () => {
        return P.string("null").result(null).desc("null");
      },
      DoubleQuoteString: () => {
        return P.regexp(/"((?:\\.|.)*?)"/, 1).desc("string");
      },
      SingleQuoteString: () => {
        return P.regexp(/'((?:\\.|.)*?)'/, 1).desc("string");
      },
      UnquotedString: () => {
        return P.regexp(/[a-zA-Z_][a-zA-Z0-9_-]*/).desc("string");
      },
      HexColor: () => {
        return P.regexp(/#(?:[0-9a-f]{3}){1,2}/i).desc("hex color");
      },
      String: (r) => {
        return P.alt(r.SingleQuoteString, r.DoubleQuoteString, r.HexColor, r.UnquotedString).map(
          (s) => s.replace(/\\n/gm, "\n")
        );
      },
      Symbol: () => {
        return P.regexp(/[a-zA-Z_][a-zA-Z0-9_-]*/).desc("symbol");
      },
      PartialSymbol: () => {
        return P.regexp(/[a-zA-Z0-9_-]+/).desc("symbol");
      },
      Feature: (r) => {
        return P.seqMap(
          P.string("@"),
          r.PartialSymbol.sepBy(P.string(".")),
          (_feat, symbols) => ["get"].concat(symbols)
        );
      },
      FeatureState: (r) => {
        return P.seqMap(
          P.string("@@"),
          r.PartialSymbol.sepBy(P.string(".")),
          (_feat, symbols) => this.expandFeatureState(symbols)
        );
      },
      Array: (r) => P.string("[").trim(r._).then(r.Value.trim(r._).sepBy(P.string(","))).skip(P.string("]")),
      State: (r) => {
        return P.seqMap(
          P.string("$"),
          r.PartialSymbol.sepBy(P.string(".")),
          (_state, symbols) => ["state"].concat(symbols)
        );
      },
      Paren: (r) => {
        return P.seqMap(P.string("("), r.Expr.trim(r._), P.string(")"), (_lb, expr, _rb) => expr);
      },
      Func: (r) => {
        return P.seqMap(
          r.Symbol.trim(r._),
          P.string("("),
          r.Expr.trim(r._).sepBy(P.string(",")),
          P.string(")"),
          (func, _lb, args, _rb) => [func, ...args]
        );
      },
      Value: (r) => {
        return P.alt(
          r.Paren,
          r.Array,
          r.Func,
          r.Boolean,
          r.Null,
          r.Number,
          r.String,
          r.FeatureState,
          r.Feature,
          r.State
        );
      },
      Expr: (r) => {
        return this.tableParser(operatorTable, r.Value);
      },
      PropertyName: () => {
        return P.regexp(/[a-zA-Z_-][a-zA-Z0-9_-]*/).desc("symbol");
      },
      Assignment: (r) => {
        return P.seqMap(
          r.PropertyName,
          P.string(":").trim(r._),
          r.Expr.trim(r._).node("expr"),
          r.End,
          (key, _colon, node) => [key, { expr: node.value, start: node.start, end: node.end }]
        );
      },
      Program: (r) => {
        return r.Assignment.trim(r._).many().map((statements) => {
          return statements.filter((s) => s !== "").reduce((acc, s) => {
            acc[s[0]] = s[1];
            return acc;
          }, {});
        });
      },
      EmptyProgram: (r) => {
        return P.seqMap(r._, P.eof, () => null);
      },
      ProgramOrEmpty: (r) => {
        return P.alt(r.EmptyProgram, r.Program);
      }
    });
  }
  /**
   * Helper function to create a parser that parsers a set of operators at the same precedence level
   *
   * @param ops Operator table
   */
  operators(ops) {
    const keys = Object.keys(ops);
    const ps = keys.map((k) => P.string(ops[k]).trim(P.optWhitespace).result(k));
    return P.alt.apply(null, ps);
  }
  /**
   * Helper function to create a table-driver operator parser
   *
   */
  tableParser(operatorTable, valueParser) {
    return operatorTable.reduce((acc, level) => level.type(level.ops, acc), valueParser);
  }
  expandFeatureState(symbols) {
    let expandedFeatureState = ["feature-state", symbols.shift()];
    symbols.forEach((s) => expandedFeatureState = ["get", s, expandedFeatureState]);
    return expandedFeatureState;
  }
}

export { ExpressionParser };
