import colorbrewer from 'colorbrewer';
import { isString, isArray } from '../../utils/type-checks.mjs';
import { FlowCoreExpressionError } from '../../errors/flow-expression-error.mjs';

const palettes = {};
function registerColorBrewerPalettes() {
  Object.keys(colorbrewer).filter((key) => key !== "schemeGroups").forEach(
    (key) => palettes[key.toLowerCase()] = {
      id: key,
      colors: colorbrewer[key][5]
    }
  );
}
function fromPalette(expr, parentExpression) {
  const paletteIdOrArray = expr[1];
  const property = expr[2];
  let colorStops = expr.slice(3);
  let palette;
  if (isString(paletteIdOrArray)) {
    palette = palettes[paletteIdOrArray.toLowerCase()]?.colors || palettes["Greys"].colors;
  } else if (isArray(paletteIdOrArray) && paletteIdOrArray[0] === "literal") {
    palette = paletteIdOrArray[1];
  } else {
    throw new FlowCoreExpressionError(parentExpression.expression, "evaluationError", [
      "from-palette requires palette id as string or custom palette array"
    ]);
  }
  const numPaletteEntries = palette.length;
  let reversePalette = false;
  if (numPaletteEntries < 5) {
    throw new FlowCoreExpressionError(parentExpression.expression, "evaluationError", [
      "custom palette requires at least 5 color entries"
    ]);
  }
  switch (colorStops.length) {
    case 2:
      break;
    case 3:
      reversePalette = !!colorStops[2];
      colorStops = colorStops.slice(0, 2);
      break;
    case numPaletteEntries:
      break;
    case numPaletteEntries + 1:
      reversePalette = !!colorStops[numPaletteEntries];
      colorStops = colorStops.slice(0, numPaletteEntries);
      break;
    default:
      throw new FlowCoreExpressionError(parentExpression.expression, "evaluationError", [
        "incorrect color stop arguments"
      ]);
  }
  if (colorStops.length === 2) {
    const min = colorStops[0];
    const max = colorStops[1];
    colorStops = [];
    const step = (max - min) / (numPaletteEntries - 1);
    for (let i = 0; i < numPaletteEntries; i++) {
      colorStops.push(min + i * step);
    }
  }
  expr = ["interpolate-lab", ["linear"], property];
  for (let i = 0; i < colorStops.length; i++) {
    expr.push(colorStops[i]);
    expr.push(reversePalette ? palette[colorStops.length - i - 1] : palette[i]);
  }
  return expr;
}
const customFromPaletteFn = {
  name: "from-palette",
  initialize: registerColorBrewerPalettes,
  fn: fromPalette
};

export { customFromPaletteFn as default };
