import * as maplibreSpec from '@emuanalytics/maplibre-gl-style-spec';
import { makeObservable, observable, computed, action } from 'mobx';
import { toStream } from 'mobx-utils';
import { FlowCoreExpressionError } from '../errors/flow-expression-error.mjs';
import { isDefined, isString } from '../utils/type-checks.mjs';
import { ExpressionParser } from './expression-parser.mjs';
import customFromPaletteFn from './functions/from-palette.mjs';
import { regex, regexi, equal, equali } from './functions/regex.mjs';
import customStateFn from './functions/state.mjs';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const _ReactiveExpression = class _ReactiveExpression {
  /**
   * Constructs a new `ReactiveExpression`.
   *
   * @example
   * ```js
   * const ex = new ReactiveExpression('1 + 2');
   * console.log(ex.value); // 3
   * ```
   *
   * @param expression - the string representation of the expression
   * @param parsedExpression - Used internally to bypass parsing when evaluating a program.
   */
  constructor(expression, parsedExpression) {
    __publicField(this, "_expression", "");
    __publicField(this, "parsedResult");
    __publicField(this, "_store", null);
    makeObservable(
      this,
      {
        _store: observable.ref,
        maplibreExpression: computed,
        value: computed,
        bindState: action
      },
      { name: this.formatName(expression) }
    );
    this._expression = expression;
    if (isDefined(parsedExpression)) {
      this.parsedResult = {
        status: true,
        value: parsedExpression
      };
    } else {
      this.parsedResult = _ReactiveExpression.parser.parseExpression(expression);
    }
  }
  /**
   * Returns `true` if the specified custom function has been registered.
   *
   * @param fn - The custom function name
   */
  static hasCustomFunction(fn) {
    return !!this.customFunctions[fn];
  }
  /**
   * Register a custom function to be used in reactive expressions.
   *
   * @param fn - The custom function to register
   */
  static registerCustomFunction(fn) {
    if (fn.initialize)
      fn.initialize();
    this.customFunctions[fn.name] = fn;
  }
  /**
   * Parses a program string into a map of properties and their associated {@link ReactiveExpression}.
   *
   * The program string should be a series of property/expression pairs separated by semicolons as follows:
   *
   * ```js
   * const program = `
   *  prop1: 1 + 2;
   *  prop2: 'a string';
   *  prop3: $stateProp * 3;
   * `;
   *
   * const parsedProgram = ReactiveExpression.parseProgram(program);
   * // {
   * //   prop1: ReactiveExpression,
   * //   prop2: ReactiveExpression,
   * //   prop3: ReactiveExpression
   * // }
   *
   * ```
   *
   * @param program - The program to parse
   * @returns A map of property names and their associated expressions.
   */
  static parseProgram(program) {
    const parsedProgram = this.parser.parseProgram(program);
    if (isString(program) && program.length === 0) {
      return {};
    }
    if (parsedProgram.status) {
      const keys = Object.keys(parsedProgram.value);
      const expressions = keys.reduce((accum, key) => {
        const expr = parsedProgram.value[key];
        const expressionString = program.slice(expr.start.offset, expr.end.offset);
        accum[key] = new _ReactiveExpression(expressionString, expr.expr);
        return accum;
      }, {});
      return expressions;
    } else {
      throw new FlowCoreExpressionError(program, "parseError", [
        `expected ${parsedProgram.expected.join(" ")} at line ${parsedProgram.index.line}, column ${parsedProgram.index.column}]`
      ]);
    }
  }
  /**
   * Generates the MapLibre-compatible representation of this expression.
   *
   * This is used internally when creating MapLibre style specifications
   * from Flo.w RDF {@link MapLayer} styles.
   *
   * This getter is a reactive `computed` and will re-evaluate within a
   * reactive context when the expression value changes.
   *
   * @computed
   *
   * @returns The MapLibre-compatible representation of this expression
   */
  get maplibreExpression() {
    if (this.isParseError(this.parsedResult)) {
      throw this.parseError;
    }
    return this.expandCustomFunctions(this.parsedResult.value);
  }
  /**
   * Evaluates the expression and returns the result.
   *
   * This getter is a reactive `computed` and will re-evaluate within a
   * reactive context when the expression value changes.
   *
   * @example
   * ```js
   * const ex = new ReactiveExpression('1 + 2');
   * console.log(ex.value); // 3
   * ```
   *
   * @computed
   *
   * @returns - The result of evaluating the expression
   */
  get value() {
    return this.evaluateExpression();
  }
  /**
   * Returns  details of parsing errors if the expression could not be parsed.
   */
  get parseError() {
    if (this.isParseError(this.parsedResult)) {
      return new FlowCoreExpressionError(this._expression, "parseError", [
        `index: ${this.parsedResult.index.offset}`,
        `expected: ${this.parsedResult.expected.join(" ")}`
      ]);
    }
    return null;
  }
  /**
   * Returns the original expression string passed to the constructor.
   */
  get expression() {
    return this._expression;
  }
  /**
   * Returns the state store bound to the expression.
   */
  get store() {
    return this._store;
  }
  /**
   * Binds a {@link StateStore} to the expression. The state store is used to resolve
   * state variables in the expression.
   *
   * Note: If no state store is bound, the expression will not be able to resolve
   * state variables and will return `null` when evaluated. A console warning will
   * be logged.
   *
   * @param store {@link StateStore} to bind to the expression
   * @returns
   */
  bindState(store) {
    this._store = store;
    return this;
  }
  /**
   * Returns a subscribable stream, which produces the value of the expression
   * whenever a reactive update causes the expression to re-evaluate.
   *
   * @example
   * ```js
   * const ex = new ReactiveExpression('$val * 2');
   * const subscription = ex.toStream().subscribe((value) => console.log(value));
   * ...
   * subscription.unsubscribe();
   * ```
   *
   * @param fireImmediately - If `true`, the stream will fire immediately with the current value.
   * If `false` the stream will only fire when the value changes.
   *
   * @returns A subscribable stream of the expression value
   */
  toStream(fireImmediately = true) {
    return toStream(() => this.value, fireImmediately);
  }
  evaluateExpression() {
    const { result, value } = maplibreSpec.expression.createExpression(this.maplibreExpression);
    if (result === "success") {
      return value.evaluate({ zoom: 0 });
    }
    throw new FlowCoreExpressionError(
      this.expression,
      "evaluationError",
      value.map((value2) => value2.message)
    );
  }
  expandCustomFunctions(expr) {
    if (this.isFunctionExpression(expr)) {
      const func = expr[0];
      const args = expr.slice(1).map((arg) => this.expandCustomFunctions(arg));
      expr = [func, ...args];
      return _ReactiveExpression.hasCustomFunction(func) ? _ReactiveExpression.customFunctions[func].fn(expr, this) : expr;
    }
    return expr;
  }
  isParseError(val) {
    return !val.status;
  }
  isFunctionExpression(expr) {
    return Array.isArray(expr);
  }
  formatName(expr) {
    return `FlowCoreExpression<${expr}>}`;
  }
};
__publicField(_ReactiveExpression, "parser", new ExpressionParser());
__publicField(_ReactiveExpression, "customFunctions", {});
let ReactiveExpression = _ReactiveExpression;
ReactiveExpression.registerCustomFunction(customStateFn);
ReactiveExpression.registerCustomFunction(customFromPaletteFn);
ReactiveExpression.registerCustomFunction(regex);
ReactiveExpression.registerCustomFunction(regexi);
ReactiveExpression.registerCustomFunction(equal);
ReactiveExpression.registerCustomFunction(equali);

export { ReactiveExpression };
