import { MapLayerStyleBuilder } from './map-layer-style-builder.mjs';

class FillLayerStyleBuilder extends MapLayerStyleBuilder {
  /**
   * Sorts features in ascending order based on this value. Features with a higher sort key will appear above features with a lower sort key.
   *
   * @param value - fill-sort-key value
   */
  fillSortKey(value) {
    this.addStyle("fill-sort-key", value);
    return this;
  }
  /**
   * Whether or not the fill should be antialiased.
   *
   * @param value - fill-antialias value (default `true`)
   */
  fillAntialias(value) {
    this.addStyle("fill-antialias", value);
    return this;
  }
  /**
   * The opacity of the entire fill layer. In contrast to the `fill-color`, this value will also affect the 1px stroke around the fill, if the stroke is used.
   *
   * @param value - fill-opacity value (default `1`)
   */
  fillOpacity(value) {
    this.addStyle("fill-opacity", value);
    return this;
  }
  /**
   * The color of the filled part of this layer. This color can be specified as `rgba` with an alpha component and the color's opacity will not affect the opacity of the 1px stroke, if it is used.
   *
   * @param value - fill-color value (default `#000000`)
   */
  fillColor(value) {
    this.addStyle("fill-color", value);
    return this;
  }
  /**
   * The outline color of the fill. Matches the value of `fill-color` if unspecified.
   *
   * @param value - fill-outline-color value
   */
  fillOutlineColor(value) {
    this.addStyle("fill-outline-color", value);
    return this;
  }
  /**
   * The geometry's offset. Values are [x, y] where negatives indicate left and up, respectively.
   *
   * @param value - fill-translate value in pixels (default `0,0`)
   */
  fillTranslate(value) {
    this.addStyle("fill-translate", value);
    return this;
  }
  /**
   * Controls the frame of reference for `fill-translate`.
   *
   * Values:
   * - map: The fill is translated relative to the map.
   * - viewport: The fill is translated relative to the viewport.
   *
   * @param value - fill-translate-anchor value (default `map`)
   */
  fillTranslateAnchor(value) {
    this.addStyle("fill-translate-anchor", value);
    return this;
  }
  /**
   * Name of image in sprite to use for drawing image fills. For seamless patterns, image width and height must be a factor of two (2, 4, 8, ..., 512). Note that zoom-dependent expressions will be evaluated only at integer zoom levels.
   *
   * @param value - fill-pattern value
   */
  fillPattern(value) {
    this.addStyle("fill-pattern", value);
    return this;
  }
}
MapLayerStyleBuilder.registerStyleBuilder("fill", FillLayerStyleBuilder);
class LineLayerStyleBuilder extends MapLayerStyleBuilder {
  /**
   * The display of line endings.
   *
   * Values:
   * - butt: A cap with a squared-off end which is drawn to the exact endpoint of the line.
   * - round: A cap with a rounded end which is drawn beyond the endpoint of the line at a radius of one-half of the line's width and centered on the endpoint of the line.
   * - square: A cap with a squared-off end which is drawn beyond the endpoint of the line at a distance of one-half of the line's width.
   *
   * @param value - line-cap value (default `butt`)
   */
  lineCap(value) {
    this.addStyle("line-cap", value);
    return this;
  }
  /**
   * The display of lines when joining.
   *
   * Values:
   * - bevel: A join with a squared-off end which is drawn beyond the endpoint of the line at a distance of one-half of the line's width.
   * - round: A join with a rounded end which is drawn beyond the endpoint of the line at a radius of one-half of the line's width and centered on the endpoint of the line.
   * - miter: A join with a sharp, angled corner which is drawn with the outer sides beyond the endpoint of the path until they meet.
   *
   * @param value - line-join value (default `miter`)
   */
  lineJoin(value) {
    this.addStyle("line-join", value);
    return this;
  }
  /**
   * Used to automatically convert miter joins to bevel joins for sharp angles.
   *
   * @param value - line-miter-limit value (default `2`)
   */
  lineMiterLimit(value) {
    this.addStyle("line-miter-limit", value);
    return this;
  }
  /**
   * Used to automatically convert round joins to miter joins for shallow angles.
   *
   * @param value - line-round-limit value (default `1.05`)
   */
  lineRoundLimit(value) {
    this.addStyle("line-round-limit", value);
    return this;
  }
  /**
   * Sorts features in ascending order based on this value. Features with a higher sort key will appear above features with a lower sort key.
   *
   * @param value - line-sort-key value
   */
  lineSortKey(value) {
    this.addStyle("line-sort-key", value);
    return this;
  }
  /**
   * The opacity at which the line will be drawn.
   *
   * @param value - line-opacity value (default `1`)
   */
  lineOpacity(value) {
    this.addStyle("line-opacity", value);
    return this;
  }
  /**
   * The color with which the line will be drawn.
   *
   * @param value - line-color value (default `#000000`)
   */
  lineColor(value) {
    this.addStyle("line-color", value);
    return this;
  }
  /**
   * The geometry's offset. Values are [x, y] where negatives indicate left and up, respectively.
   *
   * @param value - line-translate value in pixels (default `0,0`)
   */
  lineTranslate(value) {
    this.addStyle("line-translate", value);
    return this;
  }
  /**
   * Controls the frame of reference for `line-translate`.
   *
   * Values:
   * - map: The line is translated relative to the map.
   * - viewport: The line is translated relative to the viewport.
   *
   * @param value - line-translate-anchor value (default `map`)
   */
  lineTranslateAnchor(value) {
    this.addStyle("line-translate-anchor", value);
    return this;
  }
  /**
   * Stroke thickness.
   *
   * @param value - line-width value in pixels (default `1`)
   */
  lineWidth(value) {
    this.addStyle("line-width", value);
    return this;
  }
  /**
   * Draws a line casing outside of a line's actual path. Value indicates the width of the inner gap.
   *
   * @param value - line-gap-width value in pixels
   */
  lineGapWidth(value) {
    this.addStyle("line-gap-width", value);
    return this;
  }
  /**
   * The line's offset. For linear features, a positive value offsets the line to the right, relative to the direction of the line, and a negative value to the left. For polygon features, a positive value results in an inset, and a negative value results in an outset.
   *
   * @param value - line-offset value in pixels
   */
  lineOffset(value) {
    this.addStyle("line-offset", value);
    return this;
  }
  /**
   * Blur applied to the line, in pixels.
   *
   * @param value - line-blur value in pixels
   */
  lineBlur(value) {
    this.addStyle("line-blur", value);
    return this;
  }
  /**
   * Specifies the lengths of the alternating dashes and gaps that form the dash pattern. The lengths are later scaled by the line width. To convert a dash length to pixels, multiply the length by the current line width. Note that GeoJSON sources with `lineMetrics: true` specified won't render dashed lines to the expected scale. Also note that zoom-dependent expressions will be evaluated only at integer zoom levels.
   *
   * @param value - line-dasharray value in line widths
   */
  lineDasharray(value) {
    this.addStyle("line-dasharray", value);
    return this;
  }
  /**
   * Name of image in sprite to use for drawing image lines. For seamless patterns, image width must be a factor of two (2, 4, 8, ..., 512). Note that zoom-dependent expressions will be evaluated only at integer zoom levels.
   *
   * @param value - line-pattern value
   */
  linePattern(value) {
    this.addStyle("line-pattern", value);
    return this;
  }
  /**
   * Defines a gradient with which to color a line feature. Can only be used with GeoJSON sources that specify `"lineMetrics": true`.
   *
   * @param value - line-gradient value
   */
  lineGradient(value) {
    this.addStyle("line-gradient", value);
    return this;
  }
}
MapLayerStyleBuilder.registerStyleBuilder("line", LineLayerStyleBuilder);
class SymbolLayerStyleBuilder extends MapLayerStyleBuilder {
  /**
   * Label placement relative to its geometry.
   *
   * Values:
   * - point: The label is placed at the point where the geometry is located.
   * - line: The label is placed along the line of the geometry. Can only be used on `LineString` and `Polygon` geometries.
   * - line-center: The label is placed at the center of the line of the geometry. Can only be used on `LineString` and `Polygon` geometries. Note that a single feature in a vector tile may contain multiple line geometries.
   *
   * @param value - symbol-placement value (default `point`)
   */
  symbolPlacement(value) {
    this.addStyle("symbol-placement", value);
    return this;
  }
  /**
   * Distance between two symbol anchors.
   *
   * @param value - symbol-spacing value in pixels (default `250`)
   */
  symbolSpacing(value) {
    this.addStyle("symbol-spacing", value);
    return this;
  }
  /**
   * If true, the symbols will not cross tile edges to avoid mutual collisions. Recommended in layers that don't have enough padding in the vector tile to prevent collisions, or if it is a point symbol layer placed after a line symbol layer. When using a client that supports global collision detection, like MapLibre GL JS version 0.42.0 or greater, enabling this property is not needed to prevent clipped labels at tile boundaries.
   *
   * @param value - symbol-avoid-edges value
   */
  symbolAvoidEdges(value) {
    this.addStyle("symbol-avoid-edges", value);
    return this;
  }
  /**
   * Sorts features in ascending order based on this value. Features with lower sort keys are drawn and placed first.  When `icon-allow-overlap` or `text-allow-overlap` is `false`, features with a lower sort key will have priority during placement. When `icon-allow-overlap` or `text-allow-overlap` is set to `true`, features with a higher sort key will overlap over features with a lower sort key.
   *
   * @param value - symbol-sort-key value
   */
  symbolSortKey(value) {
    this.addStyle("symbol-sort-key", value);
    return this;
  }
  /**
   * Determines whether overlapping symbols in the same layer are rendered in the order that they appear in the data source or by their y-position relative to the viewport. To control the order and prioritization of symbols otherwise, use `symbol-sort-key`.
   *
   * Values:
   * - auto: Sorts symbols by `symbol-sort-key` if set. Otherwise, sorts symbols by their y-position relative to the viewport if `icon-allow-overlap` or `text-allow-overlap` is set to `true` or `icon-ignore-placement` or `text-ignore-placement` is `false`.
   * - viewport-y: Sorts symbols by their y-position relative to the viewport if `icon-allow-overlap` or `text-allow-overlap` is set to `true` or `icon-ignore-placement` or `text-ignore-placement` is `false`.
   * - source: Sorts symbols by `symbol-sort-key` if set. Otherwise, no sorting is applied; symbols are rendered in the same order as the source data.
   *
   * @param value - symbol-z-order value (default `auto`)
   */
  symbolZOrder(value) {
    this.addStyle("symbol-z-order", value);
    return this;
  }
  /**
   * If true, the icon will be visible even if it collides with other previously drawn symbols.
   *
   * @param value - icon-allow-overlap value
   */
  iconAllowOverlap(value) {
    this.addStyle("icon-allow-overlap", value);
    return this;
  }
  /**
   * Allows for control over whether to show an icon when it overlaps other symbols on the map. If `icon-overlap` is not set, `icon-allow-overlap` is used instead.
   *
   * Values:
   * - never: The icon will be hidden if it collides with any other previously drawn symbol.
   * - always: The icon will be visible even if it collides with any other previously drawn symbol.
   * - cooperative: If the icon collides with another previously drawn symbol, the overlap mode for that symbol is checked. If the previous symbol was placed using `never` overlap mode, the new icon is hidden. If the previous symbol was placed using `always` or `cooperative` overlap mode, the new icon is visible.
   *
   * @param value - icon-overlap value
   */
  iconOverlap(value) {
    this.addStyle("icon-overlap", value);
    return this;
  }
  /**
   * If true, other symbols can be visible even if they collide with the icon.
   *
   * @param value - icon-ignore-placement value
   */
  iconIgnorePlacement(value) {
    this.addStyle("icon-ignore-placement", value);
    return this;
  }
  /**
   * If true, text will display without their corresponding icons when the icon collides with other symbols and the text does not.
   *
   * @param value - icon-optional value
   */
  iconOptional(value) {
    this.addStyle("icon-optional", value);
    return this;
  }
  /**
   * In combination with `symbol-placement`, determines the rotation behavior of icons.
   *
   * Values:
   * - map: When `symbol-placement` is set to `point`, aligns icons east-west. When `symbol-placement` is set to `line` or `line-center`, aligns icon x-axes with the line.
   * - viewport: Produces icons whose x-axes are aligned with the x-axis of the viewport, regardless of the value of `symbol-placement`.
   * - auto: When `symbol-placement` is set to `point`, this is equivalent to `viewport`. When `symbol-placement` is set to `line` or `line-center`, this is equivalent to `map`.
   *
   * @param value - icon-rotation-alignment value (default `auto`)
   */
  iconRotationAlignment(value) {
    this.addStyle("icon-rotation-alignment", value);
    return this;
  }
  /**
   * Scales the original size of the icon by the provided factor. The new pixel size of the image will be the original pixel size multiplied by `icon-size`. 1 is the original size; 3 triples the size of the image.
   *
   * @param value - icon-size value in factor of the original icon size (default `1`)
   */
  iconSize(value) {
    this.addStyle("icon-size", value);
    return this;
  }
  /**
   * Scales the icon to fit around the associated text.
   *
   * Values:
   * - none: The icon is displayed at its intrinsic aspect ratio.
   * - width: The icon is scaled in the x-dimension to fit the width of the text.
   * - height: The icon is scaled in the y-dimension to fit the height of the text.
   * - both: The icon is scaled in both x- and y-dimensions.
   *
   * @param value - icon-text-fit value (default `none`)
   */
  iconTextFit(value) {
    this.addStyle("icon-text-fit", value);
    return this;
  }
  /**
   * Size of the additional area added to dimensions determined by `icon-text-fit`, in clockwise order: top, right, bottom, left.
   *
   * @param value - icon-text-fit-padding value in pixels (default `0,0,0,0`)
   */
  iconTextFitPadding(value) {
    this.addStyle("icon-text-fit-padding", value);
    return this;
  }
  /**
   * Name of image in sprite to use for drawing an image background.
   *
   * @param value - icon-image value
   */
  iconImage(value) {
    this.addStyle("icon-image", value);
    return this;
  }
  /**
   * Rotates the icon clockwise.
   *
   * @param value - icon-rotate value in degrees
   */
  iconRotate(value) {
    this.addStyle("icon-rotate", value);
    return this;
  }
  /**
   * Size of additional area round the icon bounding box used for detecting symbol collisions. Values are declared using CSS margin shorthand syntax: a single value applies to all four sides; two values apply to [top/bottom, left/right]; three values apply to [top, left/right, bottom]; four values apply to [top, right, bottom, left]. For backwards compatibility, a single bare number is accepted, and treated the same as a one-element array - padding applied to all sides.
   *
   * @param value - icon-padding value in pixels (default `2`)
   */
  iconPadding(value) {
    this.addStyle("icon-padding", value);
    return this;
  }
  /**
   * If true, the icon may be flipped to prevent it from being rendered upside-down.
   *
   * @param value - icon-keep-upright value
   */
  iconKeepUpright(value) {
    this.addStyle("icon-keep-upright", value);
    return this;
  }
  /**
   * Offset distance of icon from its anchor. Positive values indicate right and down, while negative values indicate left and up. Each component is multiplied by the value of `icon-size` to obtain the final offset in pixels. When combined with `icon-rotate` the offset will be as if the rotated direction was up.
   *
   * @param value - icon-offset value (default `0,0`)
   */
  iconOffset(value) {
    this.addStyle("icon-offset", value);
    return this;
  }
  /**
   * Part of the icon placed closest to the anchor.
   *
   * Values:
   * - center: The center of the icon is placed closest to the anchor.
   * - left: The left side of the icon is placed closest to the anchor.
   * - right: The right side of the icon is placed closest to the anchor.
   * - top: The top of the icon is placed closest to the anchor.
   * - bottom: The bottom of the icon is placed closest to the anchor.
   * - top-left: The top left corner of the icon is placed closest to the anchor.
   * - top-right: The top right corner of the icon is placed closest to the anchor.
   * - bottom-left: The bottom left corner of the icon is placed closest to the anchor.
   * - bottom-right: The bottom right corner of the icon is placed closest to the anchor.
   *
   * @param value - icon-anchor value (default `center`)
   */
  iconAnchor(value) {
    this.addStyle("icon-anchor", value);
    return this;
  }
  /**
   * Orientation of icon when map is pitched.
   *
   * Values:
   * - map: The icon is aligned to the plane of the map.
   * - viewport: The icon is aligned to the plane of the viewport.
   * - auto: Automatically matches the value of `icon-rotation-alignment`.
   *
   * @param value - icon-pitch-alignment value (default `auto`)
   */
  iconPitchAlignment(value) {
    this.addStyle("icon-pitch-alignment", value);
    return this;
  }
  /**
   * Orientation of text when map is pitched.
   *
   * Values:
   * - map: The text is aligned to the plane of the map.
   * - viewport: The text is aligned to the plane of the viewport.
   * - auto: Automatically matches the value of `text-rotation-alignment`.
   *
   * @param value - text-pitch-alignment value (default `auto`)
   */
  textPitchAlignment(value) {
    this.addStyle("text-pitch-alignment", value);
    return this;
  }
  /**
   * In combination with `symbol-placement`, determines the rotation behavior of the individual glyphs forming the text.
   *
   * Values:
   * - map: When `symbol-placement` is set to `point`, aligns text east-west. When `symbol-placement` is set to `line` or `line-center`, aligns text x-axes with the line.
   * - viewport: Produces glyphs whose x-axes are aligned with the x-axis of the viewport, regardless of the value of `symbol-placement`.
   * - viewport-glyph: When `symbol-placement` is set to `point`, aligns text to the x-axis of the viewport. When `symbol-placement` is set to `line` or `line-center`, aligns glyphs to the x-axis of the viewport and places them along the line.
   * - auto: When `symbol-placement` is set to `point`, this is equivalent to `viewport`. When `symbol-placement` is set to `line` or `line-center`, this is equivalent to `map`.
   *
   * @param value - text-rotation-alignment value (default `auto`)
   */
  textRotationAlignment(value) {
    this.addStyle("text-rotation-alignment", value);
    return this;
  }
  /**
   * Value to use for a text label. If a plain `string` is provided, it will be treated as a `formatted` with default/inherited formatting options.
   *
   * @param value - text-field value
   */
  textField(value) {
    this.addStyle("text-field", value);
    return this;
  }
  /**
   * Font stack to use for displaying text.
   *
   * @param value - text-font value (default `Open Sans Regular,Arial Unicode MS Regular`)
   */
  textFont(value) {
    this.addStyle("text-font", value);
    return this;
  }
  /**
   * Font size.
   *
   * @param value - text-size value in pixels (default `16`)
   */
  textSize(value) {
    this.addStyle("text-size", value);
    return this;
  }
  /**
   * The maximum line width for text wrapping.
   *
   * @param value - text-max-width value in ems (default `10`)
   */
  textMaxWidth(value) {
    this.addStyle("text-max-width", value);
    return this;
  }
  /**
   * Text leading value for multi-line text.
   *
   * @param value - text-line-height value in ems (default `1.2`)
   */
  textLineHeight(value) {
    this.addStyle("text-line-height", value);
    return this;
  }
  /**
   * Text tracking amount.
   *
   * @param value - text-letter-spacing value in ems
   */
  textLetterSpacing(value) {
    this.addStyle("text-letter-spacing", value);
    return this;
  }
  /**
   * Text justification options.
   *
   * Values:
   * - auto: The text is aligned towards the anchor position.
   * - left: The text is aligned to the left.
   * - center: The text is centered.
   * - right: The text is aligned to the right.
   *
   * @param value - text-justify value (default `center`)
   */
  textJustify(value) {
    this.addStyle("text-justify", value);
    return this;
  }
  /**
   * Radial offset of text, in the direction of the symbol's anchor. Useful in combination with `text-variable-anchor`, which defaults to using the two-dimensional `text-offset` if present.
   *
   * @param value - text-radial-offset value in ems
   */
  textRadialOffset(value) {
    this.addStyle("text-radial-offset", value);
    return this;
  }
  /**
   * To increase the chance of placing high-priority labels on the map, you can provide an array of `text-anchor` locations: the renderer will attempt to place the label at each location, in order, before moving onto the next label. Use `text-justify: auto` to choose justification based on anchor position. To apply an offset, use the `text-radial-offset` or the two-dimensional `text-offset`.
   *
   * Values:
   * - center: The center of the text is placed closest to the anchor.
   * - left: The left side of the text is placed closest to the anchor.
   * - right: The right side of the text is placed closest to the anchor.
   * - top: The top of the text is placed closest to the anchor.
   * - bottom: The bottom of the text is placed closest to the anchor.
   * - top-left: The top left corner of the text is placed closest to the anchor.
   * - top-right: The top right corner of the text is placed closest to the anchor.
   * - bottom-left: The bottom left corner of the text is placed closest to the anchor.
   * - bottom-right: The bottom right corner of the text is placed closest to the anchor.
   *
   * @param value - text-variable-anchor value
   */
  textVariableAnchor(value) {
    this.addStyle("text-variable-anchor", value);
    return this;
  }
  /**
        * To increase the chance of placing high-priority labels on the map, you can provide an array of `text-anchor` locations, each paired with an offset value. The renderer will attempt to place the label at each location, in order, before moving on to the next location+offset. Use `text-justify: auto` to choose justification based on anchor position.
  
       The length of the array must be even, and must alternate between enum and point entries. i.e., each anchor location must be accompanied by a point, and that point defines the offset when the corresponding anchor location is used. Positive offset values indicate right and down, while negative values indicate left and up. Anchor locations may repeat, allowing the renderer to try multiple offsets to try and place a label using the same anchor.
  
       When present, this property takes precedence over `text-anchor`, `text-variable-anchor`, `text-offset`, and `text-radial-offset`.
  
       ```json
       { "text-variable-anchor-offset": ["top", [0, 4], "left", [3,0], "bottom", [1, 1]] }
       ```
  
       When the renderer chooses the `top` anchor, `[0, 4]` will be used for `text-offset`; the text will be shifted down by 4 ems.
  
       When the renderer chooses the `left` anchor, `[3, 0]` will be used for `text-offset`; the text will be shifted right by 3 ems.
        *
        * @param value - text-variable-anchor-offset value
        */
  textVariableAnchorOffset(value) {
    this.addStyle("text-variable-anchor-offset", value);
    return this;
  }
  /**
   * Part of the text placed closest to the anchor.
   *
   * Values:
   * - center: The center of the text is placed closest to the anchor.
   * - left: The left side of the text is placed closest to the anchor.
   * - right: The right side of the text is placed closest to the anchor.
   * - top: The top of the text is placed closest to the anchor.
   * - bottom: The bottom of the text is placed closest to the anchor.
   * - top-left: The top left corner of the text is placed closest to the anchor.
   * - top-right: The top right corner of the text is placed closest to the anchor.
   * - bottom-left: The bottom left corner of the text is placed closest to the anchor.
   * - bottom-right: The bottom right corner of the text is placed closest to the anchor.
   *
   * @param value - text-anchor value (default `center`)
   */
  textAnchor(value) {
    this.addStyle("text-anchor", value);
    return this;
  }
  /**
   * Maximum angle change between adjacent characters.
   *
   * @param value - text-max-angle value in degrees (default `45`)
   */
  textMaxAngle(value) {
    this.addStyle("text-max-angle", value);
    return this;
  }
  /**
   * The property allows control over a symbol's orientation. Note that the property values act as a hint, so that a symbol whose language doesn’t support the provided orientation will be laid out in its natural orientation. Example: English point symbol will be rendered horizontally even if array value contains single 'vertical' enum value. The order of elements in an array define priority order for the placement of an orientation variant.
   *
   * Values:
   * - horizontal: If a text's language supports horizontal writing mode, symbols with point placement would be laid out horizontally.
   * - vertical: If a text's language supports vertical writing mode, symbols with point placement would be laid out vertically.
   *
   * @param value - text-writing-mode value
   */
  textWritingMode(value) {
    this.addStyle("text-writing-mode", value);
    return this;
  }
  /**
   * Rotates the text clockwise.
   *
   * @param value - text-rotate value in degrees
   */
  textRotate(value) {
    this.addStyle("text-rotate", value);
    return this;
  }
  /**
   * Size of the additional area around the text bounding box used for detecting symbol collisions.
   *
   * @param value - text-padding value in pixels (default `2`)
   */
  textPadding(value) {
    this.addStyle("text-padding", value);
    return this;
  }
  /**
   * If true, the text may be flipped vertically to prevent it from being rendered upside-down.
   *
   * @param value - text-keep-upright value (default `true`)
   */
  textKeepUpright(value) {
    this.addStyle("text-keep-upright", value);
    return this;
  }
  /**
   * Specifies how to capitalize text, similar to the CSS `text-transform` property.
   *
   * Values:
   * - none: The text is not altered.
   * - uppercase: Forces all letters to be displayed in uppercase.
   * - lowercase: Forces all letters to be displayed in lowercase.
   *
   * @param value - text-transform value (default `none`)
   */
  textTransform(value) {
    this.addStyle("text-transform", value);
    return this;
  }
  /**
   * Offset distance of text from its anchor. Positive values indicate right and down, while negative values indicate left and up. If used with text-variable-anchor, input values will be taken as absolute values. Offsets along the x- and y-axis will be applied automatically based on the anchor position.
   *
   * @param value - text-offset value in ems (default `0,0`)
   */
  textOffset(value) {
    this.addStyle("text-offset", value);
    return this;
  }
  /**
   * If true, the text will be visible even if it collides with other previously drawn symbols.
   *
   * @param value - text-allow-overlap value
   */
  textAllowOverlap(value) {
    this.addStyle("text-allow-overlap", value);
    return this;
  }
  /**
   * Allows for control over whether to show symbol text when it overlaps other symbols on the map. If `text-overlap` is not set, `text-allow-overlap` is used instead
   *
   * Values:
   * - never: The text will be hidden if it collides with any other previously drawn symbol.
   * - always: The text will be visible even if it collides with any other previously drawn symbol.
   * - cooperative: If the text collides with another previously drawn symbol, the overlap mode for that symbol is checked. If the previous symbol was placed using `never` overlap mode, the new text is hidden. If the previous symbol was placed using `always` or `cooperative` overlap mode, the new text is visible.
   *
   * @param value - text-overlap value
   */
  textOverlap(value) {
    this.addStyle("text-overlap", value);
    return this;
  }
  /**
   * If true, other symbols can be visible even if they collide with the text.
   *
   * @param value - text-ignore-placement value
   */
  textIgnorePlacement(value) {
    this.addStyle("text-ignore-placement", value);
    return this;
  }
  /**
   * If true, icons will display without their corresponding text when the text collides with other symbols and the icon does not.
   *
   * @param value - text-optional value
   */
  textOptional(value) {
    this.addStyle("text-optional", value);
    return this;
  }
  /**
   * The opacity at which the icon will be drawn.
   *
   * @param value - icon-opacity value (default `1`)
   */
  iconOpacity(value) {
    this.addStyle("icon-opacity", value);
    return this;
  }
  /**
   * The color of the icon. This can only be used with sdf icons.
   *
   * @param value - icon-color value (default `#000000`)
   */
  iconColor(value) {
    this.addStyle("icon-color", value);
    return this;
  }
  /**
   * The color of the icon's halo. Icon halos can only be used with SDF icons.
   *
   * @param value - icon-halo-color value (default `rgba(0, 0, 0, 0)`)
   */
  iconHaloColor(value) {
    this.addStyle("icon-halo-color", value);
    return this;
  }
  /**
   * Distance of halo to the icon outline.
   *
   * @param value - icon-halo-width value in pixels
   */
  iconHaloWidth(value) {
    this.addStyle("icon-halo-width", value);
    return this;
  }
  /**
   * Fade out the halo towards the outside.
   *
   * @param value - icon-halo-blur value in pixels
   */
  iconHaloBlur(value) {
    this.addStyle("icon-halo-blur", value);
    return this;
  }
  /**
   * Distance that the icon's anchor is moved from its original placement. Positive values indicate right and down, while negative values indicate left and up.
   *
   * @param value - icon-translate value in pixels (default `0,0`)
   */
  iconTranslate(value) {
    this.addStyle("icon-translate", value);
    return this;
  }
  /**
   * Controls the frame of reference for `icon-translate`.
   *
   * Values:
   * - map: Icons are translated relative to the map.
   * - viewport: Icons are translated relative to the viewport.
   *
   * @param value - icon-translate-anchor value (default `map`)
   */
  iconTranslateAnchor(value) {
    this.addStyle("icon-translate-anchor", value);
    return this;
  }
  /**
   * The opacity at which the text will be drawn.
   *
   * @param value - text-opacity value (default `1`)
   */
  textOpacity(value) {
    this.addStyle("text-opacity", value);
    return this;
  }
  /**
   * The color with which the text will be drawn.
   *
   * @param value - text-color value (default `#000000`)
   */
  textColor(value) {
    this.addStyle("text-color", value);
    return this;
  }
  /**
   * The color of the text's halo, which helps it stand out from backgrounds.
   *
   * @param value - text-halo-color value (default `rgba(0, 0, 0, 0)`)
   */
  textHaloColor(value) {
    this.addStyle("text-halo-color", value);
    return this;
  }
  /**
   * Distance of halo to the font outline. Max text halo width is 1/4 of the font-size.
   *
   * @param value - text-halo-width value in pixels
   */
  textHaloWidth(value) {
    this.addStyle("text-halo-width", value);
    return this;
  }
  /**
   * The halo's fadeout distance towards the outside.
   *
   * @param value - text-halo-blur value in pixels
   */
  textHaloBlur(value) {
    this.addStyle("text-halo-blur", value);
    return this;
  }
  /**
   * Distance that the text's anchor is moved from its original placement. Positive values indicate right and down, while negative values indicate left and up.
   *
   * @param value - text-translate value in pixels (default `0,0`)
   */
  textTranslate(value) {
    this.addStyle("text-translate", value);
    return this;
  }
  /**
   * Controls the frame of reference for `text-translate`.
   *
   * Values:
   * - map: The text is translated relative to the map.
   * - viewport: The text is translated relative to the viewport.
   *
   * @param value - text-translate-anchor value (default `map`)
   */
  textTranslateAnchor(value) {
    this.addStyle("text-translate-anchor", value);
    return this;
  }
}
MapLayerStyleBuilder.registerStyleBuilder("symbol", SymbolLayerStyleBuilder);
class CircleLayerStyleBuilder extends MapLayerStyleBuilder {
  /**
   * Sorts features in ascending order based on this value. Features with a higher sort key will appear above features with a lower sort key.
   *
   * @param value - circle-sort-key value
   */
  circleSortKey(value) {
    this.addStyle("circle-sort-key", value);
    return this;
  }
  /**
   * Circle radius.
   *
   * @param value - circle-radius value in pixels (default `5`)
   */
  circleRadius(value) {
    this.addStyle("circle-radius", value);
    return this;
  }
  /**
   * The fill color of the circle.
   *
   * @param value - circle-color value (default `#000000`)
   */
  circleColor(value) {
    this.addStyle("circle-color", value);
    return this;
  }
  /**
   * Amount to blur the circle. 1 blurs the circle such that only the centerpoint is full opacity.
   *
   * @param value - circle-blur value
   */
  circleBlur(value) {
    this.addStyle("circle-blur", value);
    return this;
  }
  /**
   * The opacity at which the circle will be drawn.
   *
   * @param value - circle-opacity value (default `1`)
   */
  circleOpacity(value) {
    this.addStyle("circle-opacity", value);
    return this;
  }
  /**
   * The geometry's offset. Values are [x, y] where negatives indicate left and up, respectively.
   *
   * @param value - circle-translate value in pixels (default `0,0`)
   */
  circleTranslate(value) {
    this.addStyle("circle-translate", value);
    return this;
  }
  /**
   * Controls the frame of reference for `circle-translate`.
   *
   * Values:
   * - map: The circle is translated relative to the map.
   * - viewport: The circle is translated relative to the viewport.
   *
   * @param value - circle-translate-anchor value (default `map`)
   */
  circleTranslateAnchor(value) {
    this.addStyle("circle-translate-anchor", value);
    return this;
  }
  /**
   * Controls the scaling behavior of the circle when the map is pitched.
   *
   * Values:
   * - map: Circles are scaled according to their apparent distance to the camera.
   * - viewport: Circles are not scaled.
   *
   * @param value - circle-pitch-scale value (default `map`)
   */
  circlePitchScale(value) {
    this.addStyle("circle-pitch-scale", value);
    return this;
  }
  /**
   * Orientation of circle when map is pitched.
   *
   * Values:
   * - map: The circle is aligned to the plane of the map.
   * - viewport: The circle is aligned to the plane of the viewport.
   *
   * @param value - circle-pitch-alignment value (default `viewport`)
   */
  circlePitchAlignment(value) {
    this.addStyle("circle-pitch-alignment", value);
    return this;
  }
  /**
   * The width of the circle's stroke. Strokes are placed outside of the `circle-radius`.
   *
   * @param value - circle-stroke-width value in pixels
   */
  circleStrokeWidth(value) {
    this.addStyle("circle-stroke-width", value);
    return this;
  }
  /**
   * The stroke color of the circle.
   *
   * @param value - circle-stroke-color value (default `#000000`)
   */
  circleStrokeColor(value) {
    this.addStyle("circle-stroke-color", value);
    return this;
  }
  /**
   * The opacity of the circle's stroke.
   *
   * @param value - circle-stroke-opacity value (default `1`)
   */
  circleStrokeOpacity(value) {
    this.addStyle("circle-stroke-opacity", value);
    return this;
  }
}
MapLayerStyleBuilder.registerStyleBuilder("circle", CircleLayerStyleBuilder);
class HeatmapLayerStyleBuilder extends MapLayerStyleBuilder {
  /**
   * Radius of influence of one heatmap point in pixels. Increasing the value makes the heatmap smoother, but less detailed.
   *
   * @param value - heatmap-radius value in pixels (default `30`)
   */
  heatmapRadius(value) {
    this.addStyle("heatmap-radius", value);
    return this;
  }
  /**
   * A measure of how much an individual point contributes to the heatmap. A value of 10 would be equivalent to having 10 points of weight 1 in the same spot. Especially useful when combined with clustering.
   *
   * @param value - heatmap-weight value (default `1`)
   */
  heatmapWeight(value) {
    this.addStyle("heatmap-weight", value);
    return this;
  }
  /**
   * Similar to `heatmap-weight` but controls the intensity of the heatmap globally. Primarily used for adjusting the heatmap based on zoom level.
   *
   * @param value - heatmap-intensity value (default `1`)
   */
  heatmapIntensity(value) {
    this.addStyle("heatmap-intensity", value);
    return this;
  }
  /**
   * Defines the color of each pixel based on its density value in a heatmap.  Should be an expression that uses `["heatmap-density"]` as input.
   *
   * @param value - heatmap-color value (default `interpolate,linear,heatmap-density,0,rgba(0, 0, 255, 0),0.1,royalblue,0.3,cyan,0.5,lime,0.7,yellow,1,red`)
   */
  heatmapColor(value) {
    this.addStyle("heatmap-color", value);
    return this;
  }
  /**
   * The global opacity at which the heatmap layer will be drawn.
   *
   * @param value - heatmap-opacity value (default `1`)
   */
  heatmapOpacity(value) {
    this.addStyle("heatmap-opacity", value);
    return this;
  }
}
MapLayerStyleBuilder.registerStyleBuilder("heatmap", HeatmapLayerStyleBuilder);
class FillExtrusionLayerStyleBuilder extends MapLayerStyleBuilder {
  /**
   * The opacity of the entire fill extrusion layer. This is rendered on a per-layer, not per-feature, basis, and data-driven styling is not available.
   *
   * @param value - fill-extrusion-opacity value (default `1`)
   */
  fillExtrusionOpacity(value) {
    this.addStyle("fill-extrusion-opacity", value);
    return this;
  }
  /**
   * The base color of the extruded fill. The extrusion's surfaces will be shaded differently based on this color in combination with the root `light` settings. If this color is specified as `rgba` with an alpha component, the alpha component will be ignored; use `fill-extrusion-opacity` to set layer opacity.
   *
   * @param value - fill-extrusion-color value (default `#000000`)
   */
  fillExtrusionColor(value) {
    this.addStyle("fill-extrusion-color", value);
    return this;
  }
  /**
   * The geometry's offset. Values are [x, y] where negatives indicate left and up (on the flat plane), respectively.
   *
   * @param value - fill-extrusion-translate value in pixels (default `0,0`)
   */
  fillExtrusionTranslate(value) {
    this.addStyle("fill-extrusion-translate", value);
    return this;
  }
  /**
   * Controls the frame of reference for `fill-extrusion-translate`.
   *
   * Values:
   * - map: The fill extrusion is translated relative to the map.
   * - viewport: The fill extrusion is translated relative to the viewport.
   *
   * @param value - fill-extrusion-translate-anchor value (default `map`)
   */
  fillExtrusionTranslateAnchor(value) {
    this.addStyle("fill-extrusion-translate-anchor", value);
    return this;
  }
  /**
   * Name of image in sprite to use for drawing images on extruded fills. For seamless patterns, image width and height must be a factor of two (2, 4, 8, ..., 512). Note that zoom-dependent expressions will be evaluated only at integer zoom levels.
   *
   * @param value - fill-extrusion-pattern value
   */
  fillExtrusionPattern(value) {
    this.addStyle("fill-extrusion-pattern", value);
    return this;
  }
  /**
   * The height with which to extrude this layer.
   *
   * @param value - fill-extrusion-height value in meters
   */
  fillExtrusionHeight(value) {
    this.addStyle("fill-extrusion-height", value);
    return this;
  }
  /**
   * The height with which to extrude the base of this layer. Must be less than or equal to `fill-extrusion-height`.
   *
   * @param value - fill-extrusion-base value in meters
   */
  fillExtrusionBase(value) {
    this.addStyle("fill-extrusion-base", value);
    return this;
  }
  /**
   * Whether to apply a vertical gradient to the sides of a fill-extrusion layer. If true, sides will be shaded slightly darker farther down.
   *
   * @param value - fill-extrusion-vertical-gradient value (default `true`)
   */
  fillExtrusionVerticalGradient(value) {
    this.addStyle("fill-extrusion-vertical-gradient", value);
    return this;
  }
}
MapLayerStyleBuilder.registerStyleBuilder("fill-extrusion", FillExtrusionLayerStyleBuilder);
class RasterLayerStyleBuilder extends MapLayerStyleBuilder {
  /**
   * The opacity at which the image will be drawn.
   *
   * @param value - raster-opacity value (default `1`)
   */
  rasterOpacity(value) {
    this.addStyle("raster-opacity", value);
    return this;
  }
  /**
   * Rotates hues around the color wheel.
   *
   * @param value - raster-hue-rotate value in degrees
   */
  rasterHueRotate(value) {
    this.addStyle("raster-hue-rotate", value);
    return this;
  }
  /**
   * Increase or reduce the brightness of the image. The value is the minimum brightness.
   *
   * @param value - raster-brightness-min value
   */
  rasterBrightnessMin(value) {
    this.addStyle("raster-brightness-min", value);
    return this;
  }
  /**
   * Increase or reduce the brightness of the image. The value is the maximum brightness.
   *
   * @param value - raster-brightness-max value (default `1`)
   */
  rasterBrightnessMax(value) {
    this.addStyle("raster-brightness-max", value);
    return this;
  }
  /**
   * Increase or reduce the saturation of the image.
   *
   * @param value - raster-saturation value
   */
  rasterSaturation(value) {
    this.addStyle("raster-saturation", value);
    return this;
  }
  /**
   * Increase or reduce the contrast of the image.
   *
   * @param value - raster-contrast value
   */
  rasterContrast(value) {
    this.addStyle("raster-contrast", value);
    return this;
  }
  /**
   * The resampling/interpolation method to use for overscaling, also known as texture magnification filter
   *
   * Values:
   * - linear: (Bi)linear filtering interpolates pixel values using the weighted average of the four closest original source pixels creating a smooth but blurry look when overscaled
   * - nearest: Nearest neighbor filtering interpolates pixel values using the nearest original source pixel creating a sharp but pixelated look when overscaled
   *
   * @param value - raster-resampling value (default `linear`)
   */
  rasterResampling(value) {
    this.addStyle("raster-resampling", value);
    return this;
  }
  /**
   * Fade duration when a new tile is added.
   *
   * @param value - raster-fade-duration value in milliseconds (default `300`)
   */
  rasterFadeDuration(value) {
    this.addStyle("raster-fade-duration", value);
    return this;
  }
}
MapLayerStyleBuilder.registerStyleBuilder("raster", RasterLayerStyleBuilder);
class HillshadeLayerStyleBuilder extends MapLayerStyleBuilder {
  /**
   * The direction of the light source used to generate the hillshading with 0 as the top of the viewport if `hillshade-illumination-anchor` is set to `viewport` and due north if `hillshade-illumination-anchor` is set to `map`.
   *
   * @param value - hillshade-illumination-direction value (default `335`)
   */
  hillshadeIlluminationDirection(value) {
    this.addStyle("hillshade-illumination-direction", value);
    return this;
  }
  /**
   * Direction of light source when map is rotated.
   *
   * Values:
   * - map: The hillshade illumination is relative to the north direction.
   * - viewport: The hillshade illumination is relative to the top of the viewport.
   *
   * @param value - hillshade-illumination-anchor value (default `viewport`)
   */
  hillshadeIlluminationAnchor(value) {
    this.addStyle("hillshade-illumination-anchor", value);
    return this;
  }
  /**
   * Intensity of the hillshade
   *
   * @param value - hillshade-exaggeration value (default `0.5`)
   */
  hillshadeExaggeration(value) {
    this.addStyle("hillshade-exaggeration", value);
    return this;
  }
  /**
   * The shading color of areas that face away from the light source.
   *
   * @param value - hillshade-shadow-color value (default `#000000`)
   */
  hillshadeShadowColor(value) {
    this.addStyle("hillshade-shadow-color", value);
    return this;
  }
  /**
   * The shading color of areas that faces towards the light source.
   *
   * @param value - hillshade-highlight-color value (default `#FFFFFF`)
   */
  hillshadeHighlightColor(value) {
    this.addStyle("hillshade-highlight-color", value);
    return this;
  }
  /**
   * The shading color used to accentuate rugged terrain like sharp cliffs and gorges.
   *
   * @param value - hillshade-accent-color value (default `#000000`)
   */
  hillshadeAccentColor(value) {
    this.addStyle("hillshade-accent-color", value);
    return this;
  }
}
MapLayerStyleBuilder.registerStyleBuilder("hillshade", HillshadeLayerStyleBuilder);
class BackgroundLayerStyleBuilder extends MapLayerStyleBuilder {
  /**
   * The color with which the background will be drawn.
   *
   * @param value - background-color value (default `#000000`)
   */
  backgroundColor(value) {
    this.addStyle("background-color", value);
    return this;
  }
  /**
   * Name of image in sprite to use for drawing an image background. For seamless patterns, image width and height must be a factor of two (2, 4, 8, ..., 512). Note that zoom-dependent expressions will be evaluated only at integer zoom levels.
   *
   * @param value - background-pattern value
   */
  backgroundPattern(value) {
    this.addStyle("background-pattern", value);
    return this;
  }
  /**
   * The opacity at which the background will be drawn.
   *
   * @param value - background-opacity value (default `1`)
   */
  backgroundOpacity(value) {
    this.addStyle("background-opacity", value);
    return this;
  }
}
MapLayerStyleBuilder.registerStyleBuilder("background", BackgroundLayerStyleBuilder);
function layerStyle(layerType) {
  return MapLayerStyleBuilder.createStyleBuilder(layerType);
}

export { BackgroundLayerStyleBuilder, CircleLayerStyleBuilder, FillExtrusionLayerStyleBuilder, FillLayerStyleBuilder, HeatmapLayerStyleBuilder, HillshadeLayerStyleBuilder, LineLayerStyleBuilder, RasterLayerStyleBuilder, SymbolLayerStyleBuilder, layerStyle };
