import * as maplibreSpec from '@emuanalytics/maplibre-gl-style-spec';
import { makeObservable, observable, computed, action } from 'mobx';
import { isDefined, isObjectFull, isNil } from '../utils/type-checks.mjs';
import { ReactiveExpression } from '../expressions/reactive-expression.mjs';
import { FlowCoreError } from '../errors/flow-error.mjs';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const _MapLayer = class _MapLayer {
  constructor(options) {
    __publicField(this, "_options");
    __publicField(this, "_state", null);
    __publicField(this, "_filter", null);
    __publicField(this, "_paintProps", {});
    __publicField(this, "_layoutProps", {});
    this._options = {
      ..._MapLayer._defaultOptions,
      ...options
    };
    this.getPropertyMetadata("visibility", options.type);
    makeObservable(
      this,
      {
        //observable
        _paintProps: observable,
        _layoutProps: observable,
        _filter: observable,
        _options: observable,
        //computed
        layerSpecification: computed,
        // actions
        bindState: action,
        setMinZoom: action,
        setMaxZoom: action,
        setInteractive: action,
        setAbove: action,
        setBelow: action,
        setStyle: action,
        setStyleProperty: action,
        setFilter: action
      },
      { name: this.formattedName(options.id) }
    );
    if (options.filter) {
      this.setFilter(options.filter);
    }
    if (options.style) {
      this.setStyle(options.style);
    }
  }
  get layerSpecification() {
    this._options.above;
    this._options.below;
    const layer = {
      id: this._options.id,
      type: this._options.type,
      source: this._options.source
    };
    if (isDefined(this._options.minZoom)) {
      layer.minzoom = this._options.minZoom;
    }
    if (isDefined(this._options.maxZoom)) {
      layer.maxzoom = this._options.maxZoom;
    }
    if (isDefined(this._options.sourceLayer)) {
      layer["source-layer"] = this._options.sourceLayer;
    }
    if (isDefined(this._filter)) {
      layer.filter = this._filter.maplibreExpression;
    }
    if (isObjectFull(this._paintProps)) {
      layer.paint = Object.keys(this._paintProps).reduce((accum, key) => {
        accum[key] = this._paintProps[key].maplibreExpression;
        return accum;
      }, {});
    }
    if (isObjectFull(this._layoutProps)) {
      layer.layout = Object.keys(this._layoutProps).reduce((accum, key) => {
        let expr = this._layoutProps[key].maplibreExpression;
        if (key === "visibility") {
          expr = this._layoutProps[key].value ? "visible" : "none";
        }
        accum[key] = expr;
        return accum;
      }, {});
    }
    return layer;
  }
  get id() {
    return this._options.id;
  }
  get type() {
    return this._options.type;
  }
  get source() {
    return this._options.source;
  }
  get interactive() {
    return !!this._options.interactive;
  }
  get above() {
    return this._options.above;
  }
  get below() {
    return this._options.below;
  }
  bindState(state) {
    this._state = state;
    if (this._filter) {
      this._filter.bindState(state);
    }
    Object.values(this._paintProps).forEach((prop) => prop.bindState(state));
    Object.values(this._layoutProps).forEach((prop) => prop.bindState(state));
    return this;
  }
  setMinZoom(zoom) {
    this._options.minZoom = zoom;
    return this;
  }
  setMaxZoom(zoom) {
    this._options.maxZoom = zoom;
    return this;
  }
  setInteractive(interactive) {
    this._options.interactive = interactive;
    return this;
  }
  setAbove(layerId) {
    this._options.above = layerId;
    return this;
  }
  setBelow(layerId) {
    this._options.below = layerId;
    return this;
  }
  setStyle(styleProgram) {
    const parsedProgram = ReactiveExpression.parseProgram(styleProgram);
    const props = Object.keys(parsedProgram);
    props.forEach((prop) => {
      const metadata = this.getPropertyMetadata(prop, this._options.type);
      const layerProps = metadata.type === "paint" ? this._paintProps : this._layoutProps;
      layerProps[prop] = parsedProgram[prop].bindState(this._state);
    });
  }
  /**
   * Set an individual style property.
   *
   * @param property Property name.
   * @param expression Property style expression.
   */
  setStyleProperty(property, expression) {
    const metadata = this.getPropertyMetadata(property, this._options.type);
    const props = metadata.type === "paint" ? this._paintProps : this._layoutProps;
    if (isNil(expression)) {
      delete props[property];
    } else {
      const expr = new ReactiveExpression(expression).bindState(this._state);
      if (expr.parseError) {
        throw expr.parseError;
      }
      props[property] = expr;
    }
  }
  setFilter(expression) {
    this._filter = null;
    if (isDefined(expression)) {
      let expr = new ReactiveExpression(expression).bindState(this._state);
      if (expr.parseError) {
        throw expr.parseError;
      }
      if (!maplibreSpec.expression.isExpressionFilter(expr.maplibreExpression)) {
        expr = new ReactiveExpression(expr.expression, ["to-boolean", expr.maplibreExpression]);
      }
      this._filter = expr;
    }
  }
  formattedName(id) {
    return `MapLayer<${id || this._options.id}>`;
  }
  getPropertyMetadata(property, layerType) {
    if (!_MapLayer._propertyMetadataIndex) {
      _MapLayer.initializePropertyMetadataIndex();
    }
    const layerMetadata = _MapLayer._propertyMetadataIndex[layerType];
    if (!layerMetadata) {
      throw new FlowCoreError(`${this.formattedName()}: unknown layer type '${layerType}'`);
    }
    const propertyMetadata = layerMetadata[property];
    if (!propertyMetadata) {
      throw new FlowCoreError(`${this.formattedName()}: unknown layer property '${property}'`);
    }
    return propertyMetadata;
  }
  static initializePropertyMetadataIndex() {
    const layerTypes = Object.keys(maplibreSpec.latest.layer.type.values);
    this._propertyMetadataIndex = layerTypes.reduce((acc, layerType) => {
      acc[layerType] = {};
      ["layout", "paint"].forEach((propType) => {
        Object.keys(maplibreSpec.latest[`${propType}_${layerType}`]).forEach((prop) => {
          acc[layerType][prop] = {
            property: prop,
            type: propType,
            definition: maplibreSpec.latest[`${propType}_${layerType}`][prop]
          };
        });
      });
      return acc;
    }, {});
  }
};
__publicField(_MapLayer, "_propertyMetadataIndex");
__publicField(_MapLayer, "_defaultOptions", {
  interactive: true,
  minZoom: 0,
  maxZoom: 18
});
let MapLayer = _MapLayer;

export { MapLayer };
