import { makeObservable, observable, computed, action } from 'mobx';
import { isNil } from '../utils/type-checks.mjs';
import { FlowCoreError } from '../errors/flow-error.mjs';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const _MapStyle = class _MapStyle {
  constructor() {
    __publicField(this, "_state", null);
    __publicField(this, "_baseStyle");
    __publicField(this, "_sources", {});
    __publicField(this, "_layers", {});
    __publicField(this, "_layerOrder", []);
    this._baseStyle = _MapStyle._defaultBaseStyle;
    makeObservable(
      this,
      {
        //observable
        _sources: observable,
        _layers: observable,
        _layerOrder: observable,
        _baseStyle: observable.ref,
        //computed
        styleSpecification: computed,
        // actions
        bindState: action,
        setBaseStyle: action,
        addSource: action,
        removeSource: action,
        addLayer: action,
        removeLayer: action
      },
      { name: this.formattedName }
    );
  }
  get styleSpecification() {
    const spec = {
      ...this._baseStyle,
      sources: { ...this._baseStyle.sources },
      layers: [...this._baseStyle.layers]
    };
    Object.values(this._sources).forEach((source) => {
      spec.sources[source.id] = source.dataSourceSpecification;
    });
    this._layerOrder.forEach((id) => {
      const layer = this._layers[id];
      const layerSpec = layer.layerSpecification;
      const baseLayerIndex = spec.layers.findIndex((l) => l.id === id);
      if (baseLayerIndex >= 0) {
        const baseLayer = spec.layers[baseLayerIndex];
        const mergedBaseLayer = {
          ...baseLayer,
          paint: {
            ...baseLayer.paint,
            ...layerSpec.paint
          },
          layout: {
            ...baseLayer.layout,
            ...layerSpec.layout
          }
        };
        spec.layers.splice(baseLayerIndex, 1, mergedBaseLayer);
      } else {
        const source = spec.sources[layer.source];
        if (source.type === "vector" && isNil(layerSpec["source-layer"])) {
          layerSpec["source-layer"] = "geometry";
        }
        const layerIndex = this.layerInsertionPoint(layer, spec);
        spec.layers.splice(layerIndex, 0, layer.layerSpecification);
      }
    });
    return spec;
  }
  get formattedName() {
    return `MapStyle<${this._baseStyle.id}>`;
  }
  bindState(state) {
    this._state = state;
    state.transaction(() => {
      Object.values(this._sources).forEach((source) => source.bindState(state));
      Object.values(this._layers).forEach((layer) => layer.bindState(state));
    });
    return this;
  }
  setBaseStyle(style) {
    this._baseStyle = style ? Object.freeze({ ...style }) : _MapStyle._defaultBaseStyle;
    return this;
  }
  addSource(source) {
    this._sources[source.id] = source;
    source.bindState(this._state);
    return this;
  }
  removeSource(id, removeLayers = false) {
    if (!this._sources[id]) {
      throw new FlowCoreError(`${this.formattedName}: no source with id '${id}'`);
    }
    const linkedLayers = Object.values(this._layers).filter((layer) => layer.source === id).map((layer) => layer.id);
    if (removeLayers) {
      linkedLayers.forEach((id2) => this.removeLayer(id2));
    } else if (linkedLayers.length) {
      throw new FlowCoreError(
        `${this.formattedName}: cannot remove source with id '${id}' when in use by layers ${linkedLayers.join(", ")}`
      );
    }
    delete this._sources[id];
    return this;
  }
  getSource(id) {
    const source = this._sources[id];
    if (!source) {
      throw new FlowCoreError(`${this.formattedName}: no source with id '${id}'`);
    }
    return source;
  }
  getSources(filter) {
    return Object.values(this._sources).filter((source) => filter ? filter(source) : true);
  }
  addLayer(layer) {
    if (!this._sources[layer.source] && !this._baseStyle.sources[layer.source]) {
      throw new FlowCoreError(`${this.formattedName}: no source with id '${layer.source}'`);
    }
    this._layers[layer.id] = layer;
    this._layerOrder.push(layer.id);
    layer.bindState(this._state);
    return this;
  }
  removeLayer(id) {
    if (!this._layers[id]) {
      throw new FlowCoreError(`${this.formattedName}: no layer with id '${id}'`);
    }
    delete this._layers[id];
    const index = this._layerOrder.indexOf(id);
    this._layerOrder.splice(index, 1);
    return this;
  }
  getLayer(id) {
    const layer = this._layers[id];
    if (!layer) {
      throw new FlowCoreError(`${this.formattedName}: no layer with id '${id}'`);
    }
    return layer;
  }
  getLayers(filter) {
    return this._layerOrder.map((layerId) => this._layers[layerId]).filter((layer) => filter ? filter(layer) : true);
  }
  layerInsertionPoint(layer, spec) {
    let insertionIndex = -1;
    if (layer.above) {
      insertionIndex = spec.layers.findIndex((l) => l.id === layer.above);
      if (insertionIndex >= 0)
        insertionIndex++;
    } else if (layer.below) {
      insertionIndex = spec.layers.findIndex((l) => l.id === layer.below);
    } else {
      const below = spec.metadata?.insertionPoints?.[layer.type];
      insertionIndex = spec.layers.findIndex((l) => l.id === below);
    }
    if (insertionIndex === -1) {
      insertionIndex = spec.layers.length;
    }
    return insertionIndex;
  }
};
__publicField(_MapStyle, "_defaultBaseStyle", Object.freeze({
  version: 8,
  id: "default",
  name: "Default base style",
  sources: {},
  layers: []
}));
let MapStyle = _MapStyle;

export { MapStyle };
