import Debug from 'debug';
import { autorun, toJS } from 'mobx';
import { isString, isURL, isArrayFull, isNil, isNull } from '../../utils/type-checks.mjs';
import { MapPluginBase } from './map-plugin-base.mjs';
import { FlowCoreError } from '../../errors/flow-error.mjs';
import { ReactiveExpression } from '../../expressions/reactive-expression.mjs';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const debug = Debug("flow:data-join-map-plugin");
const _DataJoin = class _DataJoin {
  constructor(id, map, options) {
    this.id = id;
    this.map = map;
    __publicField(this, "autorunDisposer", null);
    __publicField(this, "dataExpression", null);
    __publicField(this, "options");
    __publicField(this, "parameterExpressions", {});
    this.options = {
      ..._DataJoin.defaultOptions,
      ...options
    };
  }
  get source() {
    return this.options.source;
  }
  start() {
    this.stop();
    if (isString(this.options.data)) {
      if (isURL(this.options.data))
        this.options.data = `'${this.options.data}'`;
      this.dataExpression = new ReactiveExpression(this.options.data);
      this.dataExpression.bindState(this.map.context.state);
    } else {
      this.dataExpression = null;
    }
    if (isArrayFull(this.options.parameters)) {
      this.parameterExpressions = this.options.parameters.reduce((accum, param) => {
        const expr = new ReactiveExpression(param.value.toString());
        expr.bindState(this.map.context.state);
        accum[param.name] = expr;
        return accum;
      }, {});
    }
    this.autorunDisposer = autorun(
      async () => {
        let data = this.dataExpression ? toJS(this.dataExpression.value) : this.options.data;
        const missingParams = this.options.parameters.map((p) => p.required && isNil(this.parameterExpressions[p.name].value) ? p.name : void 0).filter((p) => !!p);
        if (isNull(data)) {
          debug(`FlowMap<${this.map.id}>: Skipping data join '${this.id}' (null data)`);
          return;
        }
        if (missingParams.length) {
          debug(
            `FlowMap<${this.map.id}>: Skipping data join '${this.id}' (null required parameters: ${missingParams.join(
              ", "
            )})`
          );
          return;
        }
        debug(`FlowMap<${this.map.id}>: Performing data join '${this.id}'`);
        if (isString(data)) {
          try {
            data = await this.loadData(data);
          } catch (e) {
            this.map.emitError(`Failed to load data for data join '${this.id}' - ${e.message}`, e);
            data = [];
          }
        }
        this.doDataJoin(data);
      },
      { name: `FlowDataJoin.${this.map.id}.${this.id})` }
    );
  }
  stop() {
    if (this.autorunDisposer) {
      this.autorunDisposer();
      this.autorunDisposer = null;
    }
  }
  doDataJoin(data) {
    const source = this.map.getSource(this.options.source);
    const sourceLayer = source.type === "vector" ? this.options.sourceLayer || "geometry" : void 0;
    if (this.options.clear) {
      this.map.removeFeatureState({ source: source.id, sourceLayer }, this.id);
    }
    const featureData = data.reduce((accum, d) => {
      accum[d[this.options.foreignKey]] = { [this.id]: d };
      return accum;
    }, {});
    this.map.setFeatureStateBulk({ source: source.id, sourceLayer }, featureData);
  }
  loadData(datasetOrUrl) {
    if (isURL(datasetOrUrl)) {
      const url = new URL(datasetOrUrl);
      const params = new URLSearchParams(url.searchParams);
      if (isArrayFull(this.options.parameters)) {
        this.options.parameters.forEach((p) => {
          params.append(p.name, this.parameterExpressions[p.name].value);
        });
      }
      url.search = params.toString();
      const headers = new Headers({ Accept: "application/json" });
      const request = new Request(url.toString(), {
        method: "GET",
        mode: "cors",
        headers,
        cache: "default"
      });
      return fetch(request).then((response) => {
        if (!response.ok) {
          throw new Error(`status=${response.status} ${response.statusText}`);
        }
        return response.json();
      });
    } else {
      const params = {};
      if (isArrayFull(this.options.parameters)) {
        this.options.parameters.forEach((p) => {
          params[p.name] = this.parameterExpressions[p.name].value;
        });
      }
      return this.map.context.flowClient.datasets.query(datasetOrUrl, {}, params);
    }
  }
};
__publicField(_DataJoin, "defaultOptions", {
  clear: false,
  parameters: []
});
let DataJoin = _DataJoin;
class DataJoinPlugin extends MapPluginBase {
  constructor(pluginManager) {
    super(pluginManager);
    __publicField(this, "_joins", /* @__PURE__ */ new Map());
    __publicField(this, "name", "DataJoinPlugin");
  }
  addDataJoin(id, options) {
    const dataJoin = new DataJoin(id, this.map, options);
    this._joins.set(id, dataJoin);
    dataJoin.start();
  }
  bindState(state) {
    this._joins.forEach((join) => join.start());
  }
  destroy() {
    this._joins.forEach((join) => join.stop());
    this._joins.clear();
  }
  handleSourceRemoved(event) {
    this._joins.forEach((join) => {
      if (join.source === event.source) {
        join.stop();
        this._joins.delete(join.id);
      }
    });
  }
  removeDataJoin(id) {
    const dataJoin = this._joins.get(id);
    if (dataJoin) {
      dataJoin.stop();
      this._joins.delete(id);
    } else {
      throw new FlowCoreError(`Cannot remove unrecognized data join '${id}'`);
    }
  }
}

export { DataJoin, DataJoinPlugin };
