import maplibre from '@emuanalytics/maplibre-gl';
import saveAs from 'file-saver';
import { randomId } from '../../../utils/random-id.mjs';
import { isFunction } from '../../../utils/type-checks.mjs';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const _MapExporter = class _MapExporter {
  constructor(map) {
    this.map = map;
    __publicField(this, "nativePixelRatio", window.devicePixelRatio);
  }
  async export(options) {
    options = { ..._MapExporter.defaultOptions, ...options };
    const [container, mapCopy] = await this.createMapCopy(options);
    return new Promise((resolve, reject) => {
      const fileName = `${options.name}.${options.format}`;
      const mapCanvas = mapCopy.getCanvas();
      const exportCanvas = this.createExportCanvas(container, mapCanvas);
      const exportContext = exportCanvas.getContext("2d");
      if (!exportContext) {
        this.destroyMapCopy(container, mapCopy);
        reject(new Error("Failed to create export canvas context"));
        return;
      }
      const drawOverlayCallback = (err) => {
        if (err) {
          this.destroyMapCopy(container, mapCopy);
          reject(err);
        } else {
          switch (options.format) {
            case "png":
              exportCanvas.toBlob((blob) => {
                blob && saveAs(blob, fileName);
              });
              break;
            case "pdf":
              throw new Error("Export to PDF is not currently supported");
          }
          this.destroyMapCopy(container, mapCopy);
          resolve(fileName);
        }
      };
      const mapImage = new Image();
      mapImage.onload = () => {
        exportContext.drawImage(mapImage, 0, 0);
        if (isFunction(options.drawOverlayFn)) {
          options.drawOverlayFn(
            exportContext,
            exportCanvas.width,
            exportCanvas.height,
            options.dpmm,
            drawOverlayCallback
          );
        } else {
          drawOverlayCallback(null);
        }
      };
      mapImage.src = mapCopy.getCanvas().toDataURL();
    });
  }
  createMapCopy(options) {
    const geoBounds = options.bounds || this.map.maplibreMap.getBounds().toArray().flat();
    const geoBottomLeft = [geoBounds[0], geoBounds[1]];
    const geoTopRight = [geoBounds[2], geoBounds[3]];
    const geoCenter = [
      (geoBottomLeft[0] + geoTopRight[0]) / 2,
      (geoBottomLeft[1] + geoTopRight[1]) / 2
    ];
    const pxBottomLeft = this.map.maplibreMap.project(geoBottomLeft);
    const pxTopRight = this.map.maplibreMap.project(geoTopRight);
    const pxWidth = pxTopRight.x - pxBottomLeft.x;
    const pxHeight = pxBottomLeft.y - pxTopRight.y;
    const exportWidth = options.width * options.dpmm;
    const exportHeight = options.height * options.dpmm;
    const aspectRatios = {
      width: pxWidth / exportWidth,
      height: pxHeight / exportHeight
    };
    const coverRatio = Math.max(aspectRatios.width, aspectRatios.height);
    const copyWidth = exportWidth * coverRatio;
    const copyHeight = exportHeight * coverRatio;
    const pixelRatio = exportWidth / copyWidth;
    this.setDevicePixelRatio(pixelRatio);
    const container = document.createElement("div");
    container.id = randomId("export");
    container.classList.add("maplibregl-export-container");
    container.style.width = `${copyWidth}px`;
    container.style.height = `${copyHeight}px`;
    this.map.container.insertBefore(container, this.map.container.firstChild);
    const mapCopy = new maplibre.Map({
      container,
      center: geoCenter,
      zoom: this.map.maplibreMap.getZoom(),
      style: { ...this.map.maplibreMap.getStyle(), transition: { delay: 0, duration: 0 } },
      bearing: this.map.maplibreMap.getBearing(),
      pitch: this.map.maplibreMap.getPitch(),
      interactive: false,
      attributionControl: false,
      fadeDuration: 0,
      preserveDrawingBuffer: true,
      transformRequest: this.transformMapRequest.bind(this)
    });
    mapCopy.on("styleimagemissing", (e) => this.handleMissingImage(e, mapCopy));
    return new Promise((resolve) => {
      mapCopy.once("load", () => {
        Object.keys(this.map.maplibreMap.style.sourceCaches).forEach((sourceId) => {
          mapCopy.style.sourceCaches[sourceId]._state.stateChanges = this.map.maplibreMap.style.sourceCaches[sourceId]._state.state;
        });
        mapCopy._update();
        setTimeout(() => resolve([container, mapCopy]), 500);
      });
    });
  }
  destroyMapCopy(container, map) {
    this.setDevicePixelRatio(this.nativePixelRatio);
    map.remove();
    container.parentNode.removeChild(container);
  }
  createExportCanvas(container, mapCanvas) {
    const canvasContainer = container.getElementsByClassName("maplibregl-canvas-container")[0];
    const overlayCanvas = document.createElement("canvas");
    overlayCanvas.id = randomId("overlay");
    overlayCanvas.width = mapCanvas.width;
    overlayCanvas.height = mapCanvas.height;
    overlayCanvas.style.position = "absolute";
    overlayCanvas.style.width = mapCanvas.style.width;
    overlayCanvas.style.height = mapCanvas.style.height;
    canvasContainer.appendChild(overlayCanvas);
    return overlayCanvas;
  }
  setDevicePixelRatio(pixelRatio) {
    Object.defineProperty(window, "devicePixelRatio", {
      get: () => pixelRatio
    });
  }
  async handleMissingImage(e, mapCopy) {
    const imageInfo = this.map.imageRegistry.getImage(e.id);
    if (imageInfo === this.map.imageRegistry.unknownImage) {
      console.error(`Export copy of map ${this.map.id}: Image '${e.id}' not registered`);
    }
    if (!imageInfo.image.complete) {
      imageInfo.image.onload = () => {
        mapCopy.updateImage(e.id, imageInfo.image);
        mapCopy.redraw();
      };
    }
    mapCopy.addImage(e.id, imageInfo.image, {
      pixelRatio: imageInfo.pixelDensity,
      sdf: imageInfo.recolor
    });
  }
  transformMapRequest(url, _resourceType) {
    if (this.map.context.flowClient.accessToken && url.startsWith(this.map.context.flowClient.baseUrl)) {
      return {
        url,
        headers: { Authorization: `Bearer ${this.map.context.flowClient.accessToken}` }
      };
    } else {
      return { url };
    }
  }
};
__publicField(_MapExporter, "defaultOptions", {
  name: "map",
  width: 297,
  height: 210,
  dpmm: 12,
  format: "png"
});
let MapExporter = _MapExporter;

export { MapExporter };
