import { a3 as MapEvent, a0 as FlowCoreMap, Y as MapSourceIdentifier, av as MapFeatureId, aw as MapPluginBase, ax as PluginManager, a5 as MapSourceEvent, a4 as MapMouseEvent, Z as MapFeatureStream, ay as PartialBy, T as MapAnimationOptions, a7 as MapStyleChangeEvent, M as MapStyleSpecification, S as StateStore, a9 as MapPopupOptions, aa as MapPopup, W as MapFeatureIdentifier } from '../../../shared/flow-rdf-core.08156446.js';
export { aC as EventHandlers, aB as PluginConstructor, aA as PluginEventsType, az as PluginOptionsType } from '../../../shared/flow-rdf-core.08156446.js';
import { F as Func } from '../../../shared/flow-rdf-core.63162669.js';
import 'eventemitter3';
import 'lit-html';
import '@emuanalytics/flow-engine-client';
import 'mobx-utils';
import '@emuanalytics/maplibre-gl';

type AnnotationMode = 'creating' | 'editing' | 'idle';
type AnnotationFeatureType = 'Polygon' | 'Circle' | 'LineString' | 'Point';
interface AnnotationStyle {
    labelSize: number;
    labelColor: string;
    labelMaxWidth: number;
    labelOffset: number[];
    labelJustify: string;
    labelAnchor: string;
    polygonLabelAnchor: string;
    radius: number;
    fillColor: string;
    strokeWidth: number;
    strokeColor: string;
    selectionStrokeColor: string;
    selectionStrokeOpacity: number;
    selectionWidth: number;
    hoverStrokeColor: string;
}
interface AnnotationFeatureProps extends AnnotationStyle {
    id: string;
    name: string;
    type: AnnotationFeatureType;
    selected: boolean;
    visible: boolean;
    labelText: string;
    [extras: string]: any;
}
interface AnnotationsServiceConfig {
    style: AnnotationStyle;
}
type AnnotationFeature = GeoJSON.Feature<GeoJSON.Geometry, Partial<AnnotationFeatureProps>> & {
    id: string;
};

interface AnnotationEventTypes {
    annotationStateChanged: (event: AnnotationEvent) => void;
}
type AnnotationEventType = keyof AnnotationEventTypes;
interface AnnotationEventData {
    features: AnnotationFeature[];
    mode: AnnotationMode;
    canUndo: boolean;
    canRedo: boolean;
    activeFeatureId: string | null;
    activeFeatureType: AnnotationFeatureType | null;
}
declare class AnnotationEvent extends MapEvent<AnnotationEventType> {
    mode: AnnotationMode;
    canUndo: boolean;
    canRedo: boolean;
    activeFeatureId: string | null;
    activeFeatureType: AnnotationFeatureType | null;
    features: AnnotationFeature[];
    constructor(type: AnnotationEventType, map: FlowCoreMap, data: AnnotationEventData);
}

type SelectionEventType = 'selectionChanged';
interface SelectionEventTypes {
    selectionChanged: (event: SelectionEvent) => void;
}
declare class SelectionEvent extends MapEvent<SelectionEventType> implements MapSourceIdentifier {
    source: string;
    sourceLayer?: string;
    features: MapFeatureId[];
    constructor(type: SelectionEventType, map: FlowCoreMap, sourceIdentifier: MapSourceIdentifier, features: MapFeatureId[]);
}

type SelectionMode = 'single' | 'multi' | 'none';
interface SelectionOptions {
    clear?: boolean | 'group';
    group?: string;
    mode: SelectionMode;
    popup?: string;
    selectionState?: string;
    sourceLayer?: string;
}
declare class SelectionPlugin extends MapPluginBase<undefined, SelectionEventTypes> {
    private _sourceSelections;
    private _enablePopups;
    readonly name = "SelectionPlugin";
    constructor(pluginManager: PluginManager);
    clearSelection(source: string): void;
    deselectFeatures(source: string, features: MapFeatureId | MapFeatureId[]): void;
    destroy(): void;
    disableSelection(source: string): void;
    enableSelection(source: string, options: SelectionOptions): void;
    handleBeforeSourceRemoved(event: MapSourceEvent): void;
    handleClick(event: MapMouseEvent): void;
    selectFeatures(source: string, features: MapFeatureId | MapFeatureId[]): void;
    selectedFeatureIds(source: string): MapFeatureId[];
    selectedFeatureStream(source: string): MapFeatureStream;
    enablePopups(enabled?: boolean): void;
    disablePopups(): void;
}

interface AnnotationOperation {
    do: Func;
    undo: Func;
}
interface AnnotationPluginOptions {
    interactive?: boolean;
    defaultStyle?: Partial<AnnotationStyle>;
    selectionOptions?: SelectionOptions;
}
declare class AnnotationPlugin extends MapPluginBase<AnnotationPluginOptions, AnnotationEventTypes> {
    readonly name = "AnnotationPlugin";
    private static _defaultOptions;
    private _features;
    private _featureIds;
    private _undoStack;
    private _redoStack;
    private _mode;
    private _activeFeatureId;
    private _activeFeatureType;
    private _onCommitCleanup;
    private _annotationDataSource;
    private _drawControl;
    private _removeKeyboardShortcuts;
    constructor(pluginManager: PluginManager, options?: AnnotationPluginOptions);
    initialize(): void;
    destroy(): void;
    clear(): this;
    undo(): this;
    /**
     * Redo the last undone operation.
     */
    redo(): this;
    getFeature(featureId: string): AnnotationFeature;
    getFeatures(): AnnotationFeature[];
    setFeatures(features: Array<PartialBy<AnnotationFeature, 'id'>>): this;
    addFeature(feature: PartialBy<AnnotationFeature, 'id'>): AnnotationFeature;
    deleteFeature(featureId: string): this;
    createFeature(featureType: AnnotationFeatureType): void;
    editFeature(featureId: string): void;
    zoomToFeature(featureId: string, animationOptions: MapAnimationOptions): this;
    setFeatureProperty(featureId: string, property: string, value: any): this;
    setFeatureProperties(featureId: string, properties: Record<string, any>): this;
    commit(): this;
    cancel(): this;
    render(): this;
    handleDblClick?(event: MapMouseEvent): void;
    handleBeforeStyleChanged(event: MapStyleChangeEvent): MapStyleSpecification;
    private startDrawing;
    private endDrawing;
    private applyDefaults;
    private generateFeatureId;
    private generateFeatureName;
    private performOperation;
    private deselectFeature;
    private deleteSelectedFeature;
    private editModeForFeature;
    private drawModeForFeatureType;
    private setFeatureGeometry;
    private isValidGeometry;
    private emitStateChange;
}

interface DataJoinOptions {
    /** Set to `true` to clear feature state before applying data join. */
    clear?: boolean;
    data: string | any[];
    /** Data join foreign key. */
    foreignKey: string;
    parameters?: DataJoinParameter[];
    source: string;
    sourceLayer?: string;
}
interface DataJoinParameter {
    name: string;
    required?: boolean;
    value: string | number | boolean;
}
declare class DataJoin {
    readonly id: string;
    readonly map: FlowCoreMap;
    private static defaultOptions;
    private autorunDisposer;
    private dataExpression;
    private options;
    private parameterExpressions;
    constructor(id: string, map: FlowCoreMap, options: DataJoinOptions);
    get source(): string;
    start(): void;
    stop(): void;
    private doDataJoin;
    private loadData;
}
/**
 *  dfsffd
 *
 *  @noInheritDoc
 *
 */
declare class DataJoinPlugin extends MapPluginBase {
    private _joins;
    readonly name = "DataJoinPlugin";
    constructor(pluginManager: PluginManager);
    addDataJoin(id: string, options: DataJoinOptions): void;
    bindState(state: StateStore): void;
    destroy(): void;
    handleSourceRemoved(event: MapSourceEvent): void;
    removeDataJoin(id: string): void;
}

/** Callback function to indicate map export overlay drawing has finished. */
type DrawOverlayCallback = (error: Error) => void;
/** Function to draw an overlay on an exported map. */
type DrawOverlayFn = (ctx: CanvasRenderingContext2D, width: number, height: number, dpmm: number, callback: DrawOverlayCallback) => void;
/**
 * Options for [[FlowCoreMap.export]].
 */
interface FlowMapExportOptions {
    /** Name of exported file. */
    name: string;
    /** Width of exported map in millimeters. */
    width: number;
    /** Height of exported map in millimeters. */
    height: number;
    /** Resolution of exported map. Dots/mm. */
    dpmm: number;
    /** Format of exported map. */
    format: 'png' | 'pdf';
    /**
     * Geographic bounds of area to export ([left, bottom, right top]).
     *
     * This should be a subregion of the full map area.
     */
    bounds?: number[];
    /** Function called to draw overlay on exported map. */
    drawOverlayFn?: DrawOverlayFn;
}

interface ExportPluginOptions {
}
declare class ExportPlugin extends MapPluginBase<ExportPluginOptions> {
    readonly name = "ExportPlugin";
    constructor(pluginManager: PluginManager);
    exportMap(options: FlowMapExportOptions): Promise<string>;
}

/**
 * This plugin supports adding hover styling to map features.
 *
 * When active, features that are hovered over will have their `hover`
 * feature state property set to `true`. This can be referenced in map layer
 * style expressions to apply hover effects.
 *
 * The plugin exposes no additional API methods or properties.
 *
 * @noInheritDoc
 *
 * @example
 * This map layer definition applies a hover effect using the `circleStrokeWidth` property:
```js{7}
  const circleLayer = new MapLayer({
    id: "circle-layer",
    type: "circle",
    source: "airquality",
    style: layerStyle("circle")
      .circleColor(expr(`@no2 ? from-palette('YlOrRd', @no2, 0, 100) : gray`))
      .circleStrokeWidth(expr("@@hover ? 5 : 0"))
      .circleRadius(expr(`@no2 ? @no2 / 5 : 5`))
      .build()
  });
```
 *
 */
declare class HoverEffectPlugin extends MapPluginBase {
    private hoveredFeature;
    readonly name = "HoverEffectPlugin";
    constructor(pluginManager: PluginManager);
    handleMouseMove(event: MapMouseEvent): void;
    handleMouseOut(event: MapEvent): void;
    handleSetExclusivePlugin(exclusivePlugins: MapPluginBase[]): void;
    private compareFeatures;
    private clearHover;
}

declare class PopupPlugin extends MapPluginBase {
    private _popupTemplates;
    private _popups;
    readonly name = "PopupPlugin";
    constructor(pluginManager: PluginManager);
    registerPopup(id: string, options: MapPopupOptions): void;
    openPopup(id: string, event: MapMouseEvent): MapPopup;
    openPopup(id: string, lngLat: [number, number], feature?: MapFeatureIdentifier): MapPopup;
    closeAllPopups(): void;
    closePopupsForSources(sources: string | string[]): void;
}

declare class UrlSyncPlugin extends MapPluginBase {
    readonly name = "UrlSyncPlugin";
    constructor(pluginManager: PluginManager);
    initialize(): void;
    handleMoveEnd(event: MapEvent): void;
    private mapStateToHash;
    private hashToMapState;
}

export { AnnotationEvent, type AnnotationEventData, type AnnotationEventType, type AnnotationEventTypes, type AnnotationFeature, type AnnotationFeatureProps, type AnnotationFeatureType, type AnnotationMode, type AnnotationOperation, AnnotationPlugin, type AnnotationPluginOptions, type AnnotationStyle, type AnnotationsServiceConfig, DataJoin, type DataJoinOptions, DataJoinPlugin, type DrawOverlayCallback, type DrawOverlayFn, ExportPlugin, type ExportPluginOptions, type FlowMapExportOptions, HoverEffectPlugin, MapPluginBase, PluginManager, PopupPlugin, SelectionEvent, type SelectionEventType, type SelectionEventTypes, type SelectionMode, type SelectionOptions, SelectionPlugin, UrlSyncPlugin };
