var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class KeyboardShortcutManager {
  constructor() {
    __publicField(this, "listeners");
    /**
     * Handles key events of the root handler and passes them down to listeners.
     * @param event
     */
    __publicField(this, "handleKeyboardEvent", (event) => {
      if (this.isInputEvent(event))
        return;
      const key = this.keyDefinitionFromEvent(event);
      for (const listener of this.listeners) {
        const handler = listener.bindings[key];
        if (handler) {
          const propagate = handler(event);
          if (propagate === true) {
            continue;
          } else {
            event.stopPropagation();
            return;
          }
        }
      }
    });
    this.listeners = [];
    document.addEventListener("keydown", this.handleKeyboardEvent);
  }
  /**
   * Adds listener to keyboard events.
   * @param bindings Object containing key/value pairs of keyCode/eventHandler function. Can add multiple comma separated `,` keyCodes for the same handler. Can optionally return `false` to stop propagation of other handlers.
   * @param priority Can optionally set the priority of event handler, higher value means higher priority. Defaults to 0.
   * @returns Function that can be used to unbind the listener.
   */
  addListener(bindings, priority) {
    bindings = this.splitEventBindings(bindings);
    const listener = this.addToListeners({ bindings, priority: priority ?? 0 });
    const removeListener = () => {
      this.removeFromListeners(listener);
    };
    return removeListener;
  }
  /**
   * Splits multiple keyCode events and creates a new bindings map.
   * @param bindings
   */
  splitEventBindings(bindings) {
    const splitBindings = {};
    for (const event in bindings) {
      event.split(",").forEach((splitEvent) => {
        splitBindings[this.normalizeKeyDefinition(splitEvent)] = bindings[event];
      });
    }
    return splitBindings;
  }
  /**
   * Checks is the event comes from an input type.
   * @param event
   */
  isInputEvent(event) {
    if (event.target instanceof Node) {
      switch (event.target.nodeName) {
        case "INPUT":
        case "SELECT":
        case "TEXTAREA":
          return true;
        default:
          return false;
      }
    }
    return false;
  }
  /**
   * Adds listener into internal array of listeners and sorts in `descending` order.
   * @param listener
   */
  addToListeners(listener) {
    this.listeners.push(listener);
    this.listeners.sort((a, b) => b.priority - a.priority);
    return listener;
  }
  /**
   * Removes listener from internal array of listeners.
   * @param listenerToRemove
   */
  removeFromListeners(listenerToRemove) {
    this.listeners = this.listeners.filter((listener) => listener !== listenerToRemove);
  }
  normalizeKeyDefinition(keyDef) {
    const parts = keyDef.split("+");
    const ev = {
      ctrlKey: parts.includes("Ctrl"),
      shiftKey: parts.includes("Shift"),
      altKey: parts.includes("Alt"),
      metaKey: parts.includes("Meta"),
      code: parts[parts.length - 1]
    };
    return this.keyDefinitionFromEvent(ev);
  }
  keyDefinitionFromEvent(ev) {
    let keyDef = "";
    if (ev.ctrlKey)
      keyDef += "Ctrl+";
    if (ev.shiftKey)
      keyDef += "Shift+";
    if (ev.altKey)
      keyDef += "Alt+";
    if (ev.metaKey)
      keyDef += "Meta+";
    keyDef += ev.code;
    return keyDef;
  }
}

export { KeyboardShortcutManager };
