import Debug from 'debug';
import { capitalize } from '../../utils/string-utils.mjs';
import { isString } from '../../utils/type-checks.mjs';
import { KeyboardShortcutManager } from './keyboard-shortcut-manager.mjs';
import { FlowCoreError } from '../../errors/flow-error.mjs';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const debug = Debug("flow:map-plugin-manager");
class PluginManager {
  constructor(map) {
    __publicField(this, "_map");
    __publicField(this, "_keyboardShortcutManager", new KeyboardShortcutManager());
    __publicField(this, "_plugins", /* @__PURE__ */ new Map());
    __publicField(this, "_initialized", false);
    __publicField(this, "_exclusivePlugins", /* @__PURE__ */ new Set());
    this._map = map;
    this.map.initialized.then(() => {
      this._initialized = true;
      for (const plugin of this._plugins.values()) {
        if (plugin.initialize) {
          plugin.initialize();
        }
      }
    });
  }
  get map() {
    return this._map;
  }
  usePlugin(plugin, options) {
    if (this._plugins.has(plugin)) {
      const oldInstance = this._plugins.get(plugin);
      if (oldInstance.destroy) {
        oldInstance.destroy();
      }
    }
    const pluginInstance = new plugin(this, options);
    this._plugins.set(plugin, pluginInstance);
    debug(`FlowMap<${this.map.id}>: Using plugin '${pluginInstance.name}'`);
    if (this._initialized && pluginInstance.initialize)
      pluginInstance.initialize();
    return pluginInstance;
  }
  getPlugin(pluginOrName) {
    let name;
    let pluginInstance;
    if (isString(pluginOrName)) {
      pluginInstance = Array.from(this._plugins.values()).find((p) => p.name === pluginOrName);
      name = pluginOrName;
    } else {
      pluginInstance = this._plugins.get(pluginOrName);
      name = pluginOrName.name;
    }
    if (!pluginInstance) {
      throw new FlowCoreError(`<${name}>: plugin not registered`);
    }
    return pluginInstance;
  }
  removePlugin(pluginOrName) {
    let pluginConstructor;
    let pluginInstance;
    if (isString(pluginOrName)) {
      const pair = Array.from(this._plugins.entries()).find(([_, p]) => p.name === pluginOrName);
      pluginConstructor = pair && pair[0];
      pluginInstance = pair && pair[1];
    } else {
      pluginConstructor = pluginOrName;
      pluginInstance = this._plugins.get(pluginOrName);
    }
    if (pluginInstance && pluginInstance.destroy) {
      pluginInstance.destroy();
    }
    if (pluginConstructor)
      this._plugins.delete(pluginConstructor);
    return this;
  }
  removeAllPlugins() {
    this._plugins.forEach((plugin) => {
      if (plugin.destroy)
        plugin.destroy();
    });
    this._plugins.clear();
    return this;
  }
  setExclusivePlugin(plugin) {
    this._exclusivePlugins.add(plugin);
    this.forwardSetExclusivePlugin();
    return this;
  }
  removeExclusivePlugin(plugin) {
    this._exclusivePlugins.delete(plugin);
    this.forwardSetExclusivePlugin();
    return this;
  }
  addKeyboardShortcuts(keyboardShortcuts, priority = 0) {
    return this._keyboardShortcutManager.addListener(keyboardShortcuts, priority);
  }
  forwardEvent(eventName, ...args) {
    const handlerName = `handle${capitalize(eventName)}`;
    this._plugins.forEach((plugin) => {
      if (this._exclusivePlugins.size && !this._exclusivePlugins.has(plugin))
        return;
      const handler = plugin[handlerName];
      if (handler) {
        handler.apply(plugin, args);
      }
    });
  }
  forwardStyleChangeEvent(event) {
    this._plugins.forEach((plugin) => {
      if (this._exclusivePlugins.size && !this._exclusivePlugins.has(plugin))
        return;
      if (plugin.handleBeforeStyleChanged) {
        const result = plugin.handleBeforeStyleChanged(event);
        if (result) {
          event.newStyle = result;
        }
      }
    });
    return event.newStyle;
  }
  forwardSetExclusivePlugin() {
    this._plugins.forEach((plugin) => {
      if (plugin.handleSetExclusivePlugin) {
        plugin.handleSetExclusivePlugin(Array.from(this._exclusivePlugins));
      }
    });
  }
}

export { PluginManager };
