import { FlowMapError } from '../../../errors/flow-map-error.mjs';
import { ensureArray } from '../../../utils/ensure-array.mjs';
import { isStringFull } from '../../../utils/type-checks.mjs';
import { MapPluginBase } from '../map-plugin-base.mjs';
import { PopupPlugin } from '../popup.plugin.mjs';
import { SourceSelection } from './source-selection.mjs';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class SelectionPlugin extends MapPluginBase {
  constructor(pluginManager) {
    super(pluginManager);
    __publicField(this, "_sourceSelections", /* @__PURE__ */ new Map());
    __publicField(this, "_enablePopups", true);
    __publicField(this, "name", "SelectionPlugin");
  }
  clearSelection(source) {
    const sourceSelection = this._sourceSelections.get(source);
    if (!sourceSelection) {
      return;
    }
    switch (sourceSelection.options.mode) {
      case "single":
      case "multi":
        sourceSelection.updateSelection(/* @__PURE__ */ new Set());
    }
  }
  deselectFeatures(source, features) {
    const sourceSelection = this._sourceSelections.get(source);
    if (!sourceSelection) {
      return;
    }
    features = ensureArray(features);
    const currentSelection = new Set(sourceSelection.selectedFeatureIds);
    switch (sourceSelection.options.mode) {
      case "single":
      case "multi":
        features.forEach((f) => currentSelection.delete(f));
        sourceSelection.updateSelection(currentSelection);
    }
  }
  destroy() {
    this._sourceSelections.forEach((sel) => sel.destroy());
    this._sourceSelections.clear();
  }
  disableSelection(source) {
    if (this._sourceSelections.has(source)) {
      this._sourceSelections.get(source).destroy();
      this._sourceSelections.delete(source);
    }
  }
  enableSelection(source, options) {
    if (options.popup) {
      try {
        this.map.getPlugin(PopupPlugin);
      } catch (e) {
        throw new FlowMapError(`enableSelection(): Popup plugin is required for automatic selection popups`);
      }
    }
    this.disableSelection(source);
    this._sourceSelections.set(source, new SourceSelection(this, source, options));
  }
  handleBeforeSourceRemoved(event) {
    if (this._sourceSelections.has(event.source)) {
      this._sourceSelections.get(event.source)?.destroy();
      this._sourceSelections.delete(event.source);
    }
  }
  handleClick(event) {
    const multiSelect = event.originalEvent.shiftKey || event.originalEvent.altKey || event.originalEvent.metaKey;
    const clickedFeature = event.features[0];
    const clickedSource = clickedFeature?.source;
    const selectionOptions = this._sourceSelections.get(clickedSource)?.options;
    this._sourceSelections.forEach(
      (sel) => sel.handleClick(event.lngLat, clickedFeature, selectionOptions, multiSelect, this._enablePopups)
    );
  }
  selectFeatures(source, features) {
    const sourceSelection = this._sourceSelections.get(source);
    if (!sourceSelection) {
      return;
    }
    features = ensureArray(features);
    if (features.length === 0) {
      this.clearSelection(source);
      return;
    }
    let currentSelection = new Set(sourceSelection.selectedFeatureIds);
    switch (sourceSelection.options.mode) {
      case "single":
        currentSelection = /* @__PURE__ */ new Set([features[0]]);
        sourceSelection.updateSelection(currentSelection);
        break;
      case "multi":
        features.forEach((f) => currentSelection.add(f));
        sourceSelection.updateSelection(currentSelection);
        break;
    }
    if (sourceSelection.selectedFeatureIds.size && isStringFull(sourceSelection.options.group)) {
      this._sourceSelections.forEach((s) => {
        if (s != sourceSelection && s.options.clear === "group" && s.options.group == sourceSelection.options.group) {
          s.updateSelection(/* @__PURE__ */ new Set());
        }
      });
    }
  }
  selectedFeatureIds(source) {
    const sourceSelection = this._sourceSelections.get(source);
    if (!sourceSelection) {
      return [];
    }
    return Array.from(sourceSelection.selectedFeatureIds);
  }
  selectedFeatureStream(source) {
    const sourceSelection = this._sourceSelections.get(source);
    if (!sourceSelection) {
      throw new FlowMapError("no source selection");
    }
    return sourceSelection.selectedFeatureStream;
  }
  enablePopups(enabled = true) {
    this._enablePopups = true;
  }
  disablePopups() {
    if (this._enablePopups) {
      this.map.getPlugin(PopupPlugin).closeAllPopups();
      this._enablePopups = false;
    }
  }
}

export { SelectionPlugin };
