import { assert } from '../../../utils/assert.mjs';
import { ensureArray } from '../../../utils/ensure-array.mjs';
import { isStringFull } from '../../../utils/type-checks.mjs';
import { isFeatureWithId } from '../../types.mjs';
import { PopupPlugin } from '../popup.plugin.mjs';
import { SelectionEvent } from './selection.events.mjs';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class SourceSelection {
  constructor(plugin, source, options) {
    __publicField(this, "_featureStream");
    __publicField(this, "_map");
    __publicField(this, "_options");
    __publicField(this, "_plugin");
    __publicField(this, "_popups", /* @__PURE__ */ new Map());
    __publicField(this, "_selectedFeatureIds", /* @__PURE__ */ new Set());
    __publicField(this, "_sourceId");
    __publicField(this, "_stateSubscription", null);
    this._plugin = plugin;
    this._map = plugin.map;
    this._options = options;
    const sourceType = plugin.map.getSource(source).type;
    this._sourceId = {
      source,
      sourceLayer: sourceType === "vector" ? options.sourceLayer ?? "geometry" : void 0
    };
    this._featureStream = this._map.featureStream({
      source,
      sourceLayer: options.sourceLayer,
      featureIds: []
    });
    if (options.selectionState) {
      this._stateSubscription = this._map.context.state.toStream(options.selectionState, true).subscribe((val) => {
        val = new Set(ensureArray(val));
        this.updateSelection(val);
      });
    }
  }
  get options() {
    return this._options;
  }
  get selectedFeatureIds() {
    return this._selectedFeatureIds;
  }
  get selectedFeatureStream() {
    return this._featureStream;
  }
  destroy() {
    this._popups.forEach((popup) => popup.close());
    this._stateSubscription?.unsubscribe();
    this._selectedFeatureIds = /* @__PURE__ */ new Set();
    this.updateMapSelectionState();
  }
  handleClick(lngLat, feature, selectionOptions, multiSelect, enablePopups) {
    if (this._options.mode === "none") {
      return;
    }
    const selectedFeatureIDs = new Set(this._selectedFeatureIds);
    let clearSelection = false;
    let toggleFeature = false;
    let showPopup = false;
    if (feature && feature.source === this._sourceId.source) {
      if (this._options.mode === "single") {
        if (selectedFeatureIDs.has(feature.id)) {
          toggleFeature = true;
        } else {
          clearSelection = true;
          toggleFeature = true;
        }
      } else {
        if (!multiSelect) {
          clearSelection = true;
        }
        toggleFeature = true;
      }
    } else if (feature && feature.source !== this._sourceId.source) {
      clearSelection = this._options.clear === "group" ? selectionOptions?.group === this._options.group : !!this._options.clear;
    } else {
      clearSelection = !multiSelect;
    }
    if (clearSelection) {
      selectedFeatureIDs.clear();
    }
    if (toggleFeature) {
      assert(isFeatureWithId(feature));
      if (selectedFeatureIDs.has(feature.id) && multiSelect) {
        selectedFeatureIDs.delete(feature.id);
        if (this._popups.has(feature.id))
          this._popups.get(feature.id).close();
      } else {
        selectedFeatureIDs.add(feature.id);
        showPopup = true;
      }
    }
    this.updateSelection(selectedFeatureIDs);
    if (enablePopups && showPopup && this._options.popup) {
      assert(isFeatureWithId(feature));
      if (!this._popups.has(feature.id)) {
        const popup = this._map.getPlugin(PopupPlugin).openPopup(this._options.popup, lngLat, feature);
        if (popup) {
          popup.on("closed", () => {
            this._popups.delete(feature.id);
          });
          this._popups.set(feature.id, popup);
        }
      }
    }
  }
  updateSelection(featureIds) {
    if (!this.areSetsEqual(featureIds, this._selectedFeatureIds)) {
      this._selectedFeatureIds = featureIds;
      this.updateMapSelectionState();
      this.updateStateStore();
      this._featureStream.featureIds = Array.from(featureIds);
      this._popups.forEach((popup) => {
        assert(isFeatureWithId(popup.feature));
        if (popup.feature.id && !featureIds.has(popup.feature.id)) {
          popup.close();
        }
      });
      const event = new SelectionEvent(
        "selectionChanged",
        this._map,
        this._sourceId,
        Array.from(this._selectedFeatureIds)
      );
      this._plugin.emit(event.type, event);
    }
  }
  areSetsEqual(a, b) {
    return a.size === b.size && [...a].every((value) => b.has(value));
  }
  updateMapSelectionState() {
    this._map.removeFeatureState(this._sourceId, "selected", false);
    this._selectedFeatureIds.forEach(
      (id) => this._map.setFeatureState({ ...this._sourceId, id }, { selected: true }, false)
    );
  }
  updateStateStore() {
    if (isStringFull(this._options.selectionState)) {
      const selectedIds = Array.from(this._selectedFeatureIds);
      if (this._options.mode === "single") {
        this._map.context.state.set(this._options.selectionState, selectedIds[0]);
      } else if (this._options.mode === "multi") {
        this._map.context.state.set(this._options.selectionState, selectedIds);
      }
    }
  }
}

export { SourceSelection };
