import Debug from 'debug';
import { makeObservable, observable, computed, onBecomeObserved, onBecomeUnobserved, runInAction } from 'mobx';
import { toStream } from 'mobx-utils';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
Debug("flow:map-feature-stream");
class MapFeatureStream {
  constructor(options) {
    __publicField(this, "_boundHandleLoading");
    __publicField(this, "_boundHandleSourceFeatureState");
    __publicField(this, "_options");
    __publicField(this, "_sourceLoadCount", 0);
    __publicField(this, "_sourceLoaded", true);
    var _a;
    this._options = {
      fireImmediately: true,
      ...options
    };
    const sourceType = options.map.getSource(options.source).type;
    if (sourceType === "vector")
      (_a = this._options).sourceLayer ?? (_a.sourceLayer = "geometry");
    makeObservable(this, {
      _options: observable,
      _sourceLoadCount: observable,
      featureData: computed.struct
    });
    this._boundHandleLoading = this.handleLoading.bind(this);
    this._boundHandleSourceFeatureState = this.handleSourceFeatureState.bind(this);
    onBecomeObserved(this, "featureData", this.resume.bind(this));
    onBecomeUnobserved(this, "featureData", this.suspend.bind(this));
  }
  /**
   * Returns feature data corresponding to the requested feature IDs. This is a computed property that will
   * automatically update when the feature data changes. Accessing this property within a reactive
   * context will automatically trigger a reactive update.
   *
   * :::info
   * This method provides an alternative to {@link subscribe} for responding to feature data updates.
   * :::
   *
   * @example
   * ```js
   * const featureStream = myMap.featureStream({
   *   featureIds: ['feature1', 'feature2'],
   *   source: 'my-source'
   * });
   *
   * const disposer = appState.autoRun(() => {
   *   const featureData = featureStream.featureData;
   *   console.log('Feature data changed:', featureData);
   * });
   * ...
   * disposer();
   * ```
   *
   */
  get featureData() {
    this._sourceLoadCount;
    const featureData = this.queryFeatureData(this._options.featureIds);
    return featureData;
  }
  /** Returns the feature IDs being tracked. */
  get featureIds() {
    return this._options.featureIds;
  }
  /** Update the feature IDs being tracked. */
  set featureIds(featureIds) {
    runInAction(() => {
      this._options.featureIds = featureIds;
    });
  }
  /**
   * Subscribe to the feature stream. The supplied handler will be called whenever the feature data changes.
   *
   * :::warning
   * Subscriptions should be disposed of when no longer required by calling their `unsubscribe()` method.
   * :::
   */
  subscribe(handler) {
    const mobxSubscription = toStream(() => this.featureData, this._options.fireImmediately).subscribe(handler);
    return {
      unsubscribe: () => {
        mobxSubscription.unsubscribe();
      }
    };
  }
  featureFromMaplibreFeature(feature) {
    return {
      id: feature.id,
      geometry: feature.geometry,
      source: this._options.source,
      sourceLayer: this._options.sourceLayer,
      properties: feature.properties,
      state: this._options.map.maplibreMap.getFeatureState({
        id: feature.id,
        source: this._options.source,
        sourceLayer: this._options.sourceLayer
      })
    };
  }
  handleLoading(event) {
    let loaded = false;
    if (event.type === "sourcedata" && event.sourceId === this._options.source) {
      loaded = event.isSourceLoaded;
    } else if (event.type === "idle") {
      loaded = true;
    }
    if (this._sourceLoaded !== loaded) {
      this._sourceLoaded = loaded;
      if (loaded) {
        runInAction(() => {
          this._sourceLoadCount++;
        });
      }
    }
  }
  handleSourceFeatureState(event) {
    if (event.type === "sourceFeatureState" && event.source === this._options.source) {
      runInAction(() => {
        this._sourceLoadCount++;
      });
    }
  }
  queryFeatureData(featureIDs) {
    if (featureIDs.length === 0) {
      return {};
    }
    const featureIdProp = this._options.map.getSource(this._options.source).featureId;
    let features = this._options.map.maplibreMap.querySourceFeatures(this._options.source, {
      sourceLayer: this._options.sourceLayer,
      filter: ["in", ["get", featureIdProp], ["literal", featureIDs]]
    });
    const ids = /* @__PURE__ */ new Set();
    features = features.filter((feature) => ids.has(feature.id) ? false : ids.add(feature.id) && true);
    const featureData = features.reduce((accum, f) => {
      accum[f.id] = this.featureFromMaplibreFeature(f);
      return accum;
    }, {});
    return featureData;
  }
  resume() {
    this._options.map.maplibreMap.on("sourcedata", this._boundHandleLoading);
    this._options.map.maplibreMap.on("idle", this._boundHandleLoading);
    this._options.map.on("sourceFeatureState", this._boundHandleSourceFeatureState);
  }
  suspend() {
    this._options.map.maplibreMap.off("sourcedata", this._boundHandleLoading);
    this._options.map.maplibreMap.off("idle", this._boundHandleLoading);
    this._options.map.off("sourceFeatureState", this._boundHandleSourceFeatureState);
  }
}

export { MapFeatureStream };
