import maplibre from '@emuanalytics/maplibre-gl';
import Debug from 'debug';
import EventEmitter from 'eventemitter3';
import * as lit from 'lit-html';
import { assert } from '../../utils/assert.mjs';
import { isArray, isStringFull, isFunction, isString } from '../../utils/type-checks.mjs';
import { isFeatureWithId } from '../types.mjs';
import { MapFeatureStream } from './map-feature-stream.mjs';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
Debug("flow:map-popup");
class MapPopup extends EventEmitter {
  /**
   * Constructs a new `MapPopup` instance.
   *
   * This is called by the `FlowMap::openPopup` method.
   *
   * @param map Parent Flo.w map.
   * @param options Popup options.
   */
  constructor(map, options) {
    super();
    this.map = map;
    this.options = options;
    __publicField(this, "_feature");
    __publicField(this, "_contentContainer");
    __publicField(this, "_featureStream", null);
    //private _featureStreamSubscription: ISubscription | null = null;
    __publicField(this, "_maplibrePopup", null);
    __publicField(this, "_featureStreamDisposer");
    this.options = {
      // Defaults
      closeButton: true,
      closeOnClick: false,
      closeOnMove: false,
      maxWidth: "300px",
      trackFeature: true,
      //
      ...this.options
    };
  }
  get feature() {
    return this._feature;
  }
  /**
   * Close the popup.
   */
  close() {
    if (this._maplibrePopup) {
      this._maplibrePopup.remove();
      this._maplibrePopup = null;
    }
  }
  /**
   * Move the popup to a new location.
   *
   * @param lngLat New location.
   */
  moveTo(lngLat) {
    this._maplibrePopup.setLngLat(lngLat);
  }
  open(eventOrLngLat, feature) {
    if (this._maplibrePopup) {
      return;
    }
    if (!isArray(eventOrLngLat)) {
      if (isFeatureWithId(eventOrLngLat.features[0])) {
        feature = eventOrLngLat.features[0];
      }
      eventOrLngLat = eventOrLngLat.lngLat;
    }
    if (feature) {
      assert(isFeatureWithId(feature), "MapPopup.open(): feature must have unique ID");
    }
    this._maplibrePopup = new maplibre.Popup({
      className: this.cssClasses().join(" "),
      closeButton: this.options.closeButton,
      closeOnClick: this.options.closeOnClick,
      closeOnMove: this.options.closeOnMove,
      maxWidth: this.options.maxWidth
    });
    this._maplibrePopup.once("close", this.popupCloseHandler.bind(this));
    this._contentContainer = document.createElement("div");
    this._contentContainer.classList.add("flow-map-popup-content");
    this._maplibrePopup.setLngLat(eventOrLngLat).setDOMContent(this._contentContainer).addTo(this.map.maplibreMap);
    if (feature) {
      this._feature = feature;
      this._featureStream = new MapFeatureStream({
        map: this.map,
        source: feature.source,
        sourceLayer: feature.sourceLayer,
        featureIds: [feature.id]
      });
      this._featureStreamDisposer = this.map.context.state.autorun(() => {
        const data = this._featureStream?.featureData;
        if (data) {
          this.handleFeatureUpdate(data[feature.id]);
        }
      });
    } else {
      this.moveTo(eventOrLngLat);
      this.renderContent(null);
    }
  }
  cssClasses() {
    const classes = ["flow-map-popup"];
    if (isStringFull(this.options.class)) {
      classes.push(this.options.class);
    }
    if (this.options.closeButton) {
      classes.push("flow-map-popup--with-close");
    }
    return classes;
  }
  handleFeatureUpdate(feature) {
    if (!feature) {
      this.close();
      return;
    }
    if (this.options.trackFeature) {
      switch (feature.geometry.type) {
        case "Point":
          this.moveTo(feature.geometry.coordinates);
          break;
        case "MultiPoint":
          this.moveTo(feature.geometry.coordinates[0]);
          break;
      }
    }
    this.renderContent(feature);
  }
  popupCloseHandler() {
    if (this._featureStreamDisposer) {
      this._featureStreamDisposer();
      this._featureStreamDisposer = void 0;
    }
    this.emit("closed", this);
    this.removeAllListeners();
  }
  renderContent(feature) {
    const state = this.map.context.state;
    const content = isFunction(this.options.content) ? this.options.content(feature, state) : this.options.content;
    if (isString(content)) {
      this._contentContainer.innerHTML = content;
    } else if (content instanceof Node) {
      this._contentContainer.innerHTML = "";
      this._contentContainer.appendChild(content);
    } else {
      lit.render(content, this._contentContainer);
    }
  }
}

export { MapPopup };
