import Debug from 'debug';
import { makeObservable, observable, onBecomeObserved, onBecomeUnobserved, reaction, runInAction } from 'mobx';
import { randomId } from '../utils/random-id.mjs';
import { AsyncCommitter } from './async-committer.mjs';
import { getGlobalState } from '../context/global-state.mjs';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const debug = Debug("flow:query");
const DEFAULT_DELAY = 100;
const _Query = class _Query {
  constructor(queryFn, transformFn, options) {
    this.queryFn = queryFn;
    __publicField(this, "_options");
    __publicField(this, "_committer", new AsyncCommitter());
    __publicField(this, "_disposer", null);
    __publicField(this, "_transformFn", (results) => results);
    __publicField(this, "results", null);
    __publicField(this, "busy", false);
    __publicField(this, "error", null);
    this._options = {
      ..._Query._defaultOptions,
      ...options
    };
    this._options.id = this._options.id ?? randomId("query");
    makeObservable(
      this,
      {
        results: observable.ref,
        busy: observable,
        error: observable.ref
      },
      {
        name: "flowQuery"
      }
    );
    onBecomeObserved(this, "results", this.wake.bind(this));
    onBecomeUnobserved(this, "results", this.sleep.bind(this));
    if (transformFn) {
      this._transformFn = transformFn;
    }
    getGlobalState().globalContextRegistry.getContext(this._options?.contextId).then((context) => {
      context.queries.registerQuery(this);
    });
  }
  static setDefaultOptions(options) {
    this._defaultOptions = { ...this._defaultOptions, ...options };
  }
  get formattedName() {
    return `Query<${this._options.id}>`;
  }
  get id() {
    return this._options.id;
  }
  destroy() {
    getGlobalState().globalContextRegistry.getContext(this._options?.contextId).then((context) => {
      context.queries.unregisterQuery(this);
    });
  }
  async runImmediately() {
    const context = await getGlobalState().globalContextRegistry.getContext(this._options.contextId);
    const queryDefinition = this.queryFn.call(context.state, context.state);
    await this.runQuery(queryDefinition);
    if (this.error) {
      throw this.error;
    }
    return this.results;
  }
  async wake() {
    this.sleep();
    const context = await getGlobalState().globalContextRegistry.getContext(this._options.contextId);
    this._disposer = reaction(
      (r) => {
        return this.queryFn.call(context.state, context.state);
      },
      (queryDefinition, prev, r) => {
        this.runQuery(queryDefinition);
      },
      { name: this.formattedName, fireImmediately: this._options.fireImmediately, delay: this._options.delay }
    );
  }
  sleep() {
    if (this._disposer) {
      runInAction(() => {
        this.results = null;
        this.error = null;
        this.busy = false;
        this._committer.abandon();
      });
      const disposer = this._disposer;
      this._disposer = null;
      disposer();
    }
  }
  async runQuery(queryDefinition) {
    debug(`${this.formattedName}: running query`, queryDefinition);
    const context = await getGlobalState().globalContextRegistry.getContext(this._options.contextId);
    try {
      if (queryDefinition === null) {
        runInAction(() => {
          this.results = null;
          this.error = null;
          this.busy = false;
        });
      } else {
        runInAction(() => {
          this.busy = true;
          this.error = null;
        });
        const response = await this._committer.run(() => {
          return context.flowClient.datasets.query(queryDefinition);
        });
        if (response.succeeded) {
          runInAction(() => {
            this.results = this._transformFn(response.value);
            this.busy = false;
          });
        } else {
        }
      }
    } catch (e) {
      console.error(`${this.formattedName}: ${e.message}`);
      runInAction(() => {
        this.results = null;
        this.error = e;
        this.busy = false;
      });
    }
  }
};
__publicField(_Query, "_defaultOptions", {
  fireImmediately: true,
  delay: DEFAULT_DELAY
});
let Query = _Query;
function query(queryFn, transformFnOrOptions, options) {
  if (typeof transformFnOrOptions === "object") {
    options = transformFnOrOptions;
    transformFnOrOptions = void 0;
  }
  return new Query(queryFn, transformFnOrOptions, options);
}
function queryMany(queryFn, transformFnOrOptions, options) {
  if (typeof transformFnOrOptions === "object") {
    options = transformFnOrOptions;
    transformFnOrOptions = void 0;
  }
  return query(queryFn, transformFnOrOptions, options);
}
function queryOne(queryFn, transformFnOrOptions, options) {
  if (typeof transformFnOrOptions === "object") {
    options = transformFnOrOptions;
    transformFnOrOptions = void 0;
  }
  const oneTransform = (results) => {
    let oneResult = null;
    if (results?.length) {
      oneResult = results[0];
    }
    if (transformFnOrOptions && typeof transformFnOrOptions === "function") {
      oneResult = transformFnOrOptions(oneResult);
    }
    return oneResult;
  };
  return query(queryFn, oneTransform, options);
}

export { Query, query, queryMany, queryOne };
